module.exports = function(grunt) {

    grunt.initConfig({
        pkg: grunt.file.readJSON('package.json'),

        jasmine: {
            pheme: {
                src: ['touch/sencha-touch.js', 'tests/spec/app.js', 'app/**/*.js', 'tests/spec/app/**/*.js'],
                options: {
                    specs: 'tests/spec/*Spec.js'
                }
            },
            coverage: {
                src: ['touch/sencha-touch.js', 'app/**/*.js' ],
                options: {
                    specs: ['tests/spec/app.js', 'tests/spec/app/**/*.js'],
                    helpers: 'tests/spec/*Helper.js',
                    template: require('grunt-template-jasmine-istanbul'),
                    templateOptions: {
                        coverage: 'tests/coverage/coverage.json',
                        report: 'tests/coverage',
                        thresholds: {
                            lines: 75,
                            statements: 75,
                            branches: 75,
                            functions: 90
                        }
                    }
                }
            }
        },

        casper: {
            options: {
                test: true
            },
            files: ['tests/integration/*.js', 'tests/integration/**/*.js']
        },

        watch: {
            files: ['*.js', 'tests/spec/app/**/*.js', 'tests/integration/**/*.js', 'tests/*.html', 'app/**/*.js', 'tests/spec/app.js'],
            tasks: ['jasmine'],
            options: {
                spawn: false
            }
        },

        jshint: {
            all: ['*.js', 'tests/*.js', 'app/*']
        },

        complexity: {
            generic: {
                src: ['app/**/*.js'],
                options: {
                    breakOnErrors: true,
                    jsLintXML: 'report.xml',         // create XML JSLint-like report
                    checkstyleXML: 'checkstyle.xml', // create checkstyle report
                    errorsOnly: false,               // show only maintainability errors
                    cyclomatic: [3, 7, 12],          // or optionally a single value, like 3
                    halstead: [8, 13, 20],           // or optionally a single value, like 8
                    maintainability: 100,
                    hideComplexFunctions: false,     // only display maintainability
                    broadcast: false                 // broadcast data over event-bus
                }
            }
        }
    });

    grunt.loadNpmTasks('grunt-contrib-jasmine');
    grunt.loadNpmTasks('grunt-contrib-watch');
    grunt.loadNpmTasks('grunt-contrib-jshint');
    grunt.loadNpmTasks('grunt-casper');
    grunt.loadNpmTasks('grunt-complexity');
    grunt.loadNpmTasks('grunt-istanbul');

    grunt.registerTask('test', ['jasmine:pheme']);
    grunt.registerTask('hint', ['jshint']);
    grunt.registerTask('complex', ['complexity']);
    grunt.registerTask('coverage', ['jasmine:coverage']);
};
