Ext.application({
    name: 'Pheme',
    description: 'Pheme was said to have pried into the affairs of mortals and gods, then repeated what she learned, starting off at first with just a dull whisper, but repeating it louder each time, until everyone knew.',
    requires: ['Ext.device.Connection', 'Ext.device.Notification', 'Ext.MessageBox', 'Ext.device.Device', 'Pheme.helpers.DfpAds', 'Pheme.helpers.UaAlerts','Pheme.helpers.Common','Pheme.helpers.ControllerEvents', 'Pheme.helpers.FavoriteStories', 'Pheme.Config', 'Pheme.helpers.UserSettings', 'Pheme.helpers.PolarAdsHelper'],
	models: ['Story', 'Topic', 'Search', 'BreakingNews', 'ReadStory', 'FavoriteStory', 'FirstUse', 'UserSettings'],
	stores: ['CentralStories', 'Stories', 'Topic', 'Search', 'BreakingNews', 'ReadStory', 'FavoriteStories', 'FirstUse', 'UserSettings'],
	views: ['About', 'Favorite', 'Main', 'Navigation', 'Search', 'StoryTitleBar', 'Story', 'Viewport', 'SearchStory', 'AdDebugSettings', 'AlertStory', 'FirstUse', 'NativeAd', 'NativeAdTitleBar'],
    controllers: ['About', 'Favorite', 'App', 'Main', 'Navigation', 'Search', 'StoryTitleBar', 'Story', 'SearchStory', 'AlertStory', 'AdDebugSettings', 'FirstUse', 'NativeAd', 'NativeAdTitleBar'],
    
    //TODO: Need to move away from using these global values. Needs refactoring.
    currentStoryIndex : 0,
    currentStoryId: 0,
    currentStoryStore: 'Stories', //Must be given a default value!
    filterActive: false,
    ads : [],
    mainViewTrackingTimeout: null,

    // time in seconds to refresh the app (e.g. read stories)
    appRefreshPeriod: 60,

    viewQueue: [],

    displayedInterstitialAd: false,
    /**
     * Flag trigger for Retina devices
     * @return {Boolean} is the device a Retina device? True/False
     */
    isRetina: function() {
        return (window.devicePixelRatio > 1) ? true : false;
    },

    icon: {
        '57': 'resources/icons/Icon.png',
        '72': 'resources/icons/Icon~ipad.png',
        '114': 'resources/icons/Icon@2x.png',
        '144': 'resources/icons/Icon~ipad@2x.png'
    },

    isIconPrecomposed: true,

    startupImage: {
        '320x460': 'resources/startup/320x460.jpg',
        '640x920': 'resources/startup/640x920.png',
        '768x1004': 'resources/startup/768x1004.png',
        '748x1024': 'resources/startup/748x1024.png',
        '1536x2008': 'resources/startup/1536x2008.png',
        '1496x2048': 'resources/startup/1496x2048.png'
    },
    /**
     * Main method for when the app is beginning execution
     * @method launch
     * method executes when the app is loaded and ready to go
     */
    launch: function() {

        //Create Loading Spinner for use in NativeAds Webviews
        Pheme.helpers.Common.createLoadingIndicator();

        var topics = Ext.getStore('topics'),
            stories = Ext.data.StoreManager.get('Stories');
        Ext.Viewport.setMaxHeight('100%'); 

        // hack to get around full size video orientation glitch
        Ext.Viewport.on('orientationchange', function(viewport, newOrientation, width, height, eOpts) {
          if('landscape' === newOrientation) {
            Ext.Viewport.setSize(height, width);
          } else {
            Ext.Viewport.setSize(width, height);
          }
        });

        // Load Topics
        topics.load();

        //We need to set the DFP Plugin Instance here so it can get the reference to the window.DFPPlugin before views are created.
        Pheme.helpers.DfpAds.setDfpPluginInstance(window.DFPPlugin);

		//Setup and init the Polar Ads SDK
		Pheme.helpers.PolarAdsHelper.initSdk(Pheme.Config.fetchCurrentMarket().polarID);

        //We need to set the PushNotifications Plugin Instance here so it can get the reference to the PushNotifications before views are created.
        try {
            Pheme.helpers.UaAlerts.setPushInst(PushNotification);
        } catch(e) {
            console.log('PushNotification not found');
        }

        //Note: use these to debug UA alert plugin 
        //Pheme.helpers.UaAlerts.setDebugMode(true);
        //Pheme.helpers.UaAlerts.setShowDebugInAlerts(true);

        //Listen for device registration for alerts
        Pheme.helpers.UaAlerts.registerDeviceRegistration(function(event, debug, alertDebug) {
            if(event) {
                if(debug) {
                    if(alertDebug) {
                        alert('Device Registered');
                    } else {
                        console.log('Device Not Registered');
                    }
                }
            }
        });
        
        // Chop off read stories store to keep things optimized. 
        try {
            var readStories = Ext.getStore("ReadStory");
            var cutOffDate = new Date();

            // keep 3 months worth of read stories
            cutOffDate.setDate(cutOffDate.getDate()-90);

            readStories.each(function(item, index, length) {
                if(item.data.date < cutOffDate)
                    readStories.remove(item);
            });
        } catch(e){ }  

        stories.load(function(self, records, successful, operation, eOpts) {
            Ext.Viewport.add(Ext.create('Pheme.view.Viewport'));
            Ext.Viewport.add(Ext.create('Pheme.view.Story'));
            Ext.Viewport.add(Ext.create('Pheme.view.Search'));
            Ext.Viewport.add(Ext.create('Pheme.view.Favorite'));
            Ext.Viewport.add(Ext.create('Pheme.view.About'));
            Ext.Viewport.add(Ext.create('Pheme.view.SearchStory'));
            Ext.Viewport.add(Ext.create('Pheme.view.AlertStory'));
            Ext.Viewport.add(Ext.create('Pheme.view.AdDebugSettings'));
            Ext.Viewport.add(Ext.create('Pheme.view.NativeAd'));
            
            // start content auto refresh
            setInterval(function(){
                Pheme.helpers.Common.autoRefreshApp();
            }, 1000 * Pheme.app.appRefreshPeriod);
            
            //Needed for Urban Airship. We need to register the event twice. On app launch and on app resume.
            //Listen for push notifications that launch the app - must be called at this point because Sencha components will exist here.
            var incomingPushCallback = function(event, debug, alertDebug, eventId) {
                if(event) {
                    if(debug) {
                        if(alertDebug) {
                            alert('Launch Push Found');
                        } else {
                           alert('No Launch Push Found');
                            console.log('Push Not Found');
                        }
                    }
                                                                
                    //If there's a url from the push notification object, we will open the story in an in-app browser. The other option is that it could be a post ID.
                    //If it's a post ID, we will open in a Story View.
                    if(event.extras.url) {
                        if(isNaN(event.extras.url)) {
                            // delay needed in order for this to work if application is launched or resumed
                            setTimeout(function() {
                                window.open(event.extras.url, "_blank", "location=yes,toolbar=yes,transitionstyle=fliphorizontal,presentationstyle=fullscreen,enableViewportScale=yes");
                            }, '1000');                             
                        } else {
							Pheme.helpers.DfpAds.removeAds();
                            var alertStoryView = Ext.getCmp('AlertStoryView');
                            
                            Pheme.app.currentStoryId = event.extras.url;

                            // TODO:  Rework breaking news store. Remove breaking news stories that have been added to the /updates feed (api)
                            Pheme.app.currentStoryStore = 'BreakingNews';

                            // TODO: Currently grabbing the latest is handled in AlertStory.js

                            // we have the story now show the AlertView
                            alertStoryView.setStoryId(Pheme.app.currentStoryId);
                            alertStoryView.setStoreId(Pheme.app.currentStoryStore);

                            if(Ext.Viewport.getActiveItem().getItemId() == 'AlertStoryView'){
                                alertStoryView.hide();
                                alertStoryView.show();
                            }

                            // don't show the alert view if first view is being displayed...queue it
                            if(Ext.Viewport.getActiveItem().id == "FirstUseView") {
                                Pheme.app.viewQueue.push(function() {
                                    Pheme.helpers.Common.switchView(alertStoryView);
                                    });
                            }
                            else {
                                Pheme.helpers.Common.switchView(alertStoryView);
                            }
                        }
                    }
                }
            };

            // Listen for push on resume
            document.addEventListener("resume", function() {
                Pheme.helpers.Common.autoRefreshApp();
                
                // refresh main stories and reset scroll to top
                var store = Ext.getStore('Stories');
                store.loadPage(1, { callback: function(records) {
                    Pheme.helpers.Common.requestNativeAd(records[0], true);
                }});
                
                Ext.getCmp('storiesList').refresh();

                Pheme.helpers.FavoriteStories.updateFavoriteUpdatedCount();

                Pheme.helpers.UaAlerts.getIncomingPush(incomingPushCallback, 'onresume');

                var topicsStore = Ext.getStore('topics');
                topicsStore.load();
            });

            // Listen for push notifications that launch the app - must be called at this point because Sencha components will exist here.
            Pheme.helpers.UaAlerts.getIncomingPush(incomingPushCallback, 'onlaunch');

            Ext.fly('appLoadingIndicator').destroy();

            // if no internet connection display message.
            var offline = document.getElementById("offline");
            if (Ext.os.is('Android')) {
                // preliminary check.. if not successful then there is most likely something wrong with the connection
                if(!successful) {
                    // need to delay this...isOnline is always true if we don't
                    setTimeout(function() {
                        if(!Ext.device.Connection.isOnline()) {
                            if(offline.style.display != "block"){
                                offline.style.display =  "block";
                                Ext.getStore('Stories').removeAll();
                            }
                        }
                        else if(offline.style.display == "block"){
                            offline.style.display =  "none";
                            Ext.getCmp("storiesList").getScrollable().getScroller().scrollTo(0, 0);
                            Ext.getStore("topics").loadPage(1);
                        }
                    }, '1000'); 
                }
            }
            else {
                if(navigator.onLine)
                    offline.style.display="none";
                else  
                    offline.style.display="block";
            }
            
            Pheme.helpers.FavoriteStories.updateFavoriteUpdatedCount();

            // set up a listener to handle the back button for Android 
            if (Ext.os.is('Android')) {
                document.addEventListener("backbutton", Ext.bind(onBackKeyDown, this), false);  // add back button listener
                function onBackKeyDown(e) {
                    
                    e.preventDefault();
                    var view = Ext.Viewport.getActiveItem();
                    var main  = Ext.getCmp('MainView').element;
                    var alertPopup = Ext.getCmp('FavoriteView');

                    if(!alertPopup.isHidden()) {
                        alertPopup.hide();
                    }
                    else if (view.xtype == 'Viewport' ) {
                        var filterBar = Ext.getCmp('filter_bar');
                        var list = Ext.getCmp('storiesList');

                        if(main.hasCls('out') || main.hasCls('out-no-animation')){
                            Pheme.helpers.Common.switchElement(main);
                            list.getScrollable().getScroller().setDisabled(false);
                            Pheme.helpers.DfpAds.renderBannerAd(list.getStore().first(), '#252525', true, true);
                        }
                        else if(filterBar && !filterBar.isHidden()){ //is in filtered view
                            var store = Ext.getStore("Stories");
                            store.getProxy().setUrl(Pheme.Config.fetchCurrentMarket().endpoint + '/stories'),
                            store.loadPage(1),
                            Pheme.app.currentStoryStore = 'Stories';
                            filterBar.hide();
                            list.getScrollable().getScroller().setDisabled(false);
                            list.getScrollable().getScroller().scrollTo(0, 0);
                            list.removeCls("filtered");
                        }
                        else
                            navigator.app.exitApp();
                    }
                    else if (Ext.Viewport.getActiveItem().id == 'SearchStoryView' ) {
                        Pheme.helpers.Common.switchView('#SearchView');
                    }
                    else {
                        Pheme.helpers.DfpAds.removeAds();
                        Pheme.helpers.Common.switchView('#mainViewport');

                        if(main.hasCls('out') || main.hasCls('out-no-animation')) {
                            Pheme.helpers.Common.switchElement(main, false, false);
                            storiesList.getScrollable().getScroller().setDisabled(false);
                        }

                        Pheme.helpers.DfpAds.renderBannerAd(Ext.getCmp('storiesList').getStore().first(), '#252525', true, true);
                    }
                    Pheme.helpers.Omniture.pageEvent("Device Back Button", "");
                }

                document.addEventListener("menubutton", function() {
                    var view = Ext.Viewport.getActiveItem();
                    var main  = Ext.getCmp('MainView').element;
                    var alertPopup = Ext.getCmp('FavoriteView');
                    var storiesList = Ext.getCmp('storiesList');

                    if(!alertPopup.isHidden())
                        alertPopup.hide();

                    if (view.xtype == 'Viewport' ) {
                        if (main.hasCls('out') || main.hasCls('out-no-animation')) {
                            Pheme.helpers.Common.switchElement(main);

                            storiesList.getScrollable().getScroller().setDisabled(false);
                            Pheme.helpers.DfpAds.renderBannerAd(storiesList.getStore().first(), '#252525', true, true);
                        } 
                        else {
                            Pheme.helpers.Common.switchElement(main, true);
                            
                            storiesList.getScrollable().getScroller().setDisabled(true);  
                            Pheme.helpers.DfpAds.removeAds();
                        }
                    }
                    else {
                        Pheme.helpers.Common.switchView('#mainViewport');

                        Pheme.helpers.Common.switchElement(main, true);

                        storiesList.getScrollable().getScroller().setDisabled(true);  
                        Pheme.helpers.DfpAds.removeAds();
                    }

                    Pheme.helpers.Omniture.pageEvent("Device Menu Button", "");
                }, false);
            }
            
            // show status bar         
            try{
                StatusBar.show();
                StatusBar.overlaysWebView(true);
            } catch(e) {}       

            Pheme.app.showFirstUse();
        });

    },
    /**
     * First Use method.  Shows app tutorial carousel
     * @method showFirstUse
     * Method executes when the app is launched for the first time
     */
    showFirstUse: function() {
        // FIRST USE CAROUSEL
        // TODO: Only show this carousel if this is the first time ever that the user has started this app
        var firstUseStore = Ext.data.StoreManager.get('FirstUse');
        firstUseStore.load();

        var hasBeenUsedBefore = firstUseStore.findExact('isFirstTime', false);
                 
        if(hasBeenUsedBefore < 0) {
            var firstUseCarousel = Ext.create('Pheme.view.FirstUse');

            var fucPanel = Ext.create('Ext.Panel', {
                layout: 'fit',
                id: 'firstView',
                items: [
                    {
                        xtype: 'panel',
                        cls: 'first-page',
                        html:'<div class="f-wrapper">' +
                            '<div class="mid-content">' +
                            '<p class="message width-first">Welcome <br />The ' + Pheme.Config.fetchCurrentMarket().aboutUs + ' app delivers smart, fast and entertaining news and information as it happens. It\'s news with personality, produced precisely for you on your mobile phone.</p>' +
                            '</div>',
                },
                ]
            });

            firstUseCarousel.add(fucPanel);

            fucPanel = Ext.create('Ext.Panel', {
                layout: 'fit',
                id: 'secondView',
                items: [
                    {
                        xtype: 'panel',
                        cls: 'first-page',
                        html:'<div class="f-wrapper">' +
                             '<div class="balloon-arrow"></div>' +
                             '<div class="follow-balloon">Added to Following</div>' +
                             '<div class="top-buttons">' +
                             '<div class="back-button"></div><div class="alert"></div><div class="follow"></div><div class="select"></div>' +
                             '</div>' +
                             '<div class="mock-image"></div>' +
                             '<div class="content-table">' +
                             '<div class="content-cell">' +
                             '<p class="follow-message-header">Interested in a developing story?<p>' +
                             '<p class="follow-message">Tap the star icon to follow stories and receive updates as they happen.</p>' +
                             '</div>' +
                             '</div>' +
                             '</div>'
                    }
                ]
            });
                 
            firstUseCarousel.add(fucPanel);
            
            fucPanel = Ext.create('Ext.Panel', {
                layout: 'fit',
                id: 'thirdView',
                items: [
                    {
                        xtype: 'panel',
                        cls: 'first-page',
                        html:'<div class="f-wrapper">' +
                            '<div class="grad-top">' +
                            '<div class="logo">' +
                            '<p>' + Pheme.Config.fetchCurrentMarket().aboutUs + ' <span>&nbsp;&nbsp;&nbsp;now</span><br />Man shot in the forehead with a handgun. <br />He is alive and refusing to talk to police.</p>' +
                            '</div>' +
                            '</div>' +
                            '<div class="content-block">' +
                            '<div class="mid-content">' +
                            '<p class="message width-second">All the latest news<br />' +
                            'Notifications for your followed stories will be delivered as soon as they\'re updated with new information.</p>' +
                            '</div>' +
                            '</div>' +
                            '</div>'
                    }
                ]
            });

            firstUseCarousel.add(fucPanel);

            fucPanel = Ext.create('Ext.Panel', {
                docked: 'bottom',
                items: [
                    {
                        xtype: 'image',
                        cls: 'bottom-content',
                        itemId: 'nav_start',
                        html:   '<div class="bottom-content">' +
                            '<div class="get-started-button">' +
                            'GET STARTED' +
                            '</div>' +
                            '</div>',
                    }
                ]
            });
            
            firstUseCarousel.add(fucPanel);

            Pheme.helpers.DfpAds.removeAds();

            Ext.Viewport.setActiveItem(firstUseCarousel);

            firstUseStore.add({ 'isFirstTime': false });
            
            firstUseStore.sync();
			
			//Only call AdWords if running on iOS
			if (Ext.os.is.iOS) {
				// Options for Google AdWords Conversion Tracking
				var options = {
			    		conversionId: Pheme.Config.fetchCurrentMarket().googleAdwordsConversionId,
			            label: Pheme.Config.fetchCurrentMarket().googleAdwordsConversionLabel.download,
			            value: "0",
			            repeatable: false
			        };
			
            	//Plugin can be found at: https://github.com/Postmedia-Digital/cordova-google-adwords	
        		CordovaGoogleAdWords.track(options, null, null);
			}

        }
    },
    /**
     * Trigger for successful update to the app while open
     * @method onUpdated
     * Method executes when the app is updated successfully, prompting the user to restart
     */
    onUpdated: function() {
        Ext.Msg.confirm("Application Update", "This application has just successfully been updated to the latest version. Reload now?", function(buttonId) {
            if (buttonId === 'yes') {
                window.location.reload();
            }
        });
    }
});
