/**
 * Config
 * @class Pheme.Config
 * Description: This configuration file manages the different texts and defaults 
 * found in each of the markets.  To help minimize the differences
 * between each market app, any references to a city or paper should
 * refer to a field defined in this config. 
 */
Ext.define('Pheme.Config', {
    singleton: true,

    config: {
        currentMarket: 'calgary',
        appVers: '1.1.8',
        nativeAdSlot: 4,

        markets: {
            ottawa: {
                resourcesDirectory: 'ottawa',
                endpoint: 'http://staging-smrtphone.herokuapp.com/oc-staging',
                adCode: 'oc',
                distributor: 'ottawacitizen',
                newspaperBrand : 'ottawacitizen',
                aboutUs : 'Ottawa Citizen',
                paperWebsite: 'ottawacitizen.com',
				googleAdwordsConversionId: '969729260',
				googleAdwordsConversionLabel: {
					download: 'kFjfCK_UkVYQ7MmzzgM' // ios
				},
                polarID: 'NA-OTTACITI-11236588'
            },
            montreal: {
                resourcesDirectory: 'montreal',
                endpoint: 'http://smrtphone.canada.com/montreal',
                adCode: 'mg',
                distributor: 'montrealgazette',
                newspaperBrand : 'montrealgazette',
                aboutUs : 'Montreal Gazette',
                paperWebsite: 'montrealgazette.com',
				googleAdwordsConversionId: '959202989',
				googleAdwordsConversionLabel: {
					download: 'zJIdCJ7l11YQrY2xyQM' // ios
				},
                polarID: 'NA-MONTGAZE-11236589'
            },
            calgary: {
                resourcesDirectory: 'calgary',
                endpoint: 'http://smrtphone.canada.com/calgary',
                adCode: 'ch',
                distributor: 'calgaryherald',
                newspaperBrand : 'calgaryherald',
                aboutUs : 'Calgary Herald',
                paperWebsite: 'calgaryherald.com',
				googleAdwordsConversionId: '962407348',
				googleAdwordsConversionLabel: {
					download: '4yzsCN3D11YQtNf0ygM' // ios
				},
                polarID: 'NA-CALGHERA-11236590'
            },
            saskatoonstarphoenix: {
                resourcesDirectory: 'saskatoonstarphoenix',
                endpoint: '',
                adCode: '',
                distributor: 'starphoenix',
                newspaperBrand : 'starphoenix',
                aboutUs : '',
                paperWebsite: '',
				googleAdwordsConversionId: '',
				googleAdwordsConversionLabel: {
					download: '' 
				},
                polarID: 'NA-STARPHOE-11236788'
            },
            reginaleaderpost: {
                resourcesDirectory: 'reginaleaderpost',
                endpoint: '',
                adCode: '',
                distributor: 'leaderpost',
                newspaperBrand : 'leaderpost',
                aboutUs : '',
                paperWebsite: '',
				googleAdwordsConversionId: '',
				googleAdwordsConversionLabel: {
					download: '' 
				},
                polarID: 'NA-LEADPOST-11236819'
            },
            vancouversun: {
                resourcesDirectory: 'vancouversun',
                endpoint: '',
                adCode: '',
                distributor: 'vancouversun',
                newspaperBrand : 'vancouversun',
                aboutUs : '',
                paperWebsite: '',
				googleAdwordsConversionId: '',
				googleAdwordsConversionLabel: {
					download: '' 
				},
                polarID: 'NA-VANCSUN-11236822'
            },
            edmontonjournal: {
                resourcesDirectory: 'edmontonjournal',
                endpoint: 'http://smrtphone.canada.com/edmontonjournal',
                adCode: 'ej',
                distributor: 'edmontonjournal',
                newspaperBrand : 'edmontonjournal',
                aboutUs : 'Edmonton Journal',
                paperWebsite: 'edmontonjournal.com',
				googleAdwordsConversionId: '966756134',
                googleAdwordsConversionLabel: {
                    download: 'VFiZCJPl5lgQpo7-zAM' 
                },
                polarID: 'NA-EDMOJOUR-11236818'
            },
            windsorstar: {
                resourcesDirectory: 'windsorstar',
                endpoint: 'http://smrtphone.canada.com/windsorstar',
                adCode: 'ws',
                distributor: 'windsorstar',
                newspaperBrand : 'windsorstar',
                aboutUs : 'Windsor Star',
                paperWebsite: 'windsorstar.com',
                googleAdwordsConversionId: '952970778',
                googleAdwordsConversionLabel: {
                    download: 'zH1wCNiW5FgQmty0xgM' 
                },
                polarID: 'NA-WINDSTAR-11236820'
            }
        }
    },

    /**
     * Find the currentMarket variable
     * @return {string} the current Market (city name)
     */
    fetchCurrentMarket: function() {
        var currentMarket = this.getCurrentMarket();

        return this.getMarkets()[currentMarket];
    },
    /**
     * Find the current app Version number
     * @return {string} current app version number
     */
    fetchVersionNum: function(){
        return this.getAppVers();
    },
    /**
     * loads the config object for building
     * @param  {obj} config the current config object
     */
    constructor: function(config) {
        this.initConfig(config);
    },
    /**
     * init method call
     */
    init: function() {
        this.callParent(arguments);
    },
    currentYear: function(){
        var currentTime = new Date();
        return currentTime.getFullYear();
    }
});