/**
 *  @class Pheme.controller.Alert
 *  @extends Ext.app.Controller
 *  Description: Controller for the Breaking news story alerts
 */
Ext.define('Pheme.controller.Favorite', {
    extend: 'Ext.app.Controller',
    config: {
        refs: {
            popup: '#FavoriteView',
            favoriteList: '#FavoriteList'
        },
        control: {
            favoriteList: {
                itemtap: function(list, index, target, record, e) {
                    var activeView = Ext.Viewport.getActiveItem();
                    var storyId = record.data.id;
                    if (!record.data.breaking) {
                        var targetHtml = target.innerHtmlElement.dom.children[1].classList,
                            unfollowStory = false;
                        for (var i = 0, len = targetHtml.length; i < len; i++) {
                            if (targetHtml[i] === 'follow-story-item-left') {
                                unfollowStory = true;
                            }
                        }
                        if (unfollowStory && e.target.classList[0] === 'unfollow-button') {
                            if (Pheme.helpers.FavoriteStories.isFavoriteStory(storyId)) {

                                Pheme.helpers.FavoriteStories.toggleFavoriteStory(storyId);

                                var followedStoryCount = Pheme.helpers.FavoriteStories.favoriteCount();
                                
                                // update count within the favorite popup
                                var followingCmp = Ext.getCmp("followingCount");
                                followingCmp.setHtml(followedStoryCount);

                                // no favorite stories - display how-to message
                                if(followedStoryCount == 0) {
                                    Ext.getCmp('FavoriteView').addCls('breaking-news-no-stories');
                                }
                                else {
                                    Ext.getCmp('FavoriteView').removeCls('breaking-news-no-stories');
                                }

                                var viewId = activeView.getItemId()
                                if (viewId == 'StoryView' || viewId == 'SearchStoryView' || viewId == 'AlertStoryView') {
                                    var activeItem = activeView.getActiveItem();
                                    if (activeItem && activeItem.internalId && activeItem.internalId == storyId) {
                                        Ext.ComponentQuery.query('#btn_follow').forEach(function(followButton) {
                                            followButton.setSrc('resources/images/toolbar/follow-btn-wh.png');
                                        });
                                        var innerFollowButton = Ext.ComponentQuery.query('#' + activeItem.getItemId() + ' #innerStoryFollowButton')[0];
                                        if (innerFollowButton) {
                                            innerFollowButton.removeCls('inner-following');
                                        }
                                    }
                                } else if (activeView.getItemId() == 'mainViewport') {
                                    Ext.getCmp('storiesList').refresh();
                                }
                                try {
                                    Pheme.helpers.UaAlerts.removeTag(record.data.id.toString());
                                } catch (e) {}

                                Pheme.helpers.Omniture.storyAction("Unfollow Swipe", storyId);
                            } else if (activeView.getItemId() == 'mainViewport') {
                                Ext.getCmp('storiesList').refresh();
                            }
                        } else if (unfollowStory) {
                            //This accounts for the user tapping the story while the unfollow button is visible.
                            //In this case, we just want to hide the unfollow button
                            targetHtml.add('follow-story-item-right');
                            targetHtml.remove('follow-story-item-left');
                        } else {
                            Pheme.helpers.Omniture.storyAction("Followed Content Click", storyId);

                            this.showStoryView(storyId, 'Stories');
                        }
                    } else {
                        Pheme.helpers.Omniture.storyAction("Alert Content Click", storyId, { storyStore: 'BreakingNews' });
                    
                        //If the story has a content field, this means it's a story from WordPress and we can handle it normally.
                        if (record.data.content) {
                            this.showStoryView(storyId, 'Stories');
                        } else {
                            //This is in the event that the story has a link field, which means that it is a Breaking News Alert link to a story on the website.
                            window.open(record.data.link, "_blank", "location=yes,toolbar=yes,transitionstyle=fliphorizontal,presentationstyle=fullscreen,enableViewportScale=yes");
                        }
                    }
                },
                itemswipe: function(self, index, target, record, e, eOpts) {
                    var targetHtml = target.innerHtmlElement.dom.children[1].classList;
                    if (!record.data.breaking) {
                        if (e.direction === 'left') {
                            // remove all unfollow buttons from the list
                            var items = Ext.query('#' + self.container.id + ' div.follow-story-item-left');
                            for (var i = 0; i < items.length; i++) {
                                var element = Ext.get(items[i]);
                                element.addCls('follow-story-item-right');
                                element.removeCls('follow-story-item-left');
                            }
                            // add button to this story
                            targetHtml.remove('follow-story-item-right');
                            targetHtml.add('follow-story-item-left');
                        } else if (e.direction === 'right' && targetHtml.contains('follow-story-item-left')) {
                            // remove button from this story
                            targetHtml.add('follow-story-item-right');
                            targetHtml.remove('follow-story-item-left');
                        }
                    }
                }
            }
        }
    },
    /**
     * [showStoryView description]
     * @param  {[type]} storyId [description]
     * @param  {[type]} store   [description]
     * @return {[type]}         [description]
     */
	showStoryView: function(storyId, store) {
    	var alertWindow = this.getPopup(),
    		activeViewportItem = Ext.Viewport.getActiveItem(),
			alertStoryView = Ext.getCmp('AlertStoryView');

		Pheme.app.currentStoryId = storyId;
		Pheme.app.currentStoryStore = store;

        alertStoryView.setStoryId(storyId);
        alertStoryView.setStoreId(store);
        
        alertWindow.isHidden() ? alertWindow.show() : alertWindow.hide();
        
		if(Ext.Viewport.getActiveItem().getItemId() == 'AlertStoryView'){
			alertStoryView.hide();
			alertStoryView.show();
		}

        Pheme.helpers.Common.switchView('#AlertStoryView');
    }
});
