/**
 *  @class Pheme.controller.GenericStory
 *  @extends Ext.app.Controller
 *  Description: Controller for the Generic Story screen
 */
Ext.define('Pheme.controller.GenericStory', {
	extend: 'Ext.app.Controller',
	adIndex: 0,
    eventsQueue: [],
    /**
     * Adds story for viewing
     * @param {obj} story The Story object of the desired Story.
     */
	addStory: function (story) {
		var carousel = this.customCarouselFactory(story.data.id),
            carouselPanel = this.carouselPanelFactory(),
            contentPanel = this.generateContentPanel(story);

        this.addContentPanelsToCarousel(story, carouselPanel, contentPanel);

        if(this.checkIfFeaturedStoryExists(story) && !this.checkIfGallery(story)) {
            var featuredImage = this.imagePanelFactory(story, Pheme.app.isRetina());

            if(this.checkIfCreditExists(story)) {
                contentPanel.insert(0, this.imageCaptionPanelFactory(story));
            }

            carouselPanel.insert(0, featuredImage);

        } else {
            contentPanel.setDocked('');
        }

		carousel.add(carouselPanel);

        this.addStoryContentPanelsToCarousel(carousel, story.data.content);

        var genericStoryClass = this;
        carousel.on({
            activeitemchange: {
                fn: function () {

                    var currentEventInQueue,
                        argumentsForEventInQueue,
                        referenceToParentCarousel = this;

                    for (var i = 0; i < genericStoryClass.eventsQueue.length; i++) {

                        currentEventInQueue = genericStoryClass.eventsQueue[i];
                        argumentsForEventInQueue = currentEventInQueue.args;
                        argumentsForEventInQueue.push(referenceToParentCarousel);

                        if ('activeitemchange' === currentEventInQueue.eventTrigger && referenceToParentCarousel.parent._itemId === currentEventInQueue.caller) {
                            currentEventInQueue.functionToRun.call(genericStoryClass, argumentsForEventInQueue);
                        }
                    }
                }
            }
        });

        this.setCarouselProperties(carousel, story);

        return carousel;
	},
    /**
     * Adds Advertisement to story 
     * @param  {int} sid Advert identifier
     * @return {config}     Returns the configuration object for the interstitial ad for the story.
     */
	getAd: function (sid) {
		return Ext.create('Ext.Panel', {
			aliases: 'adPanel',
			//zIndex: 5,
			items: [{
				xtype: 'image',
				cls: 'story_ad',
				id: 'InterstitialAd',
				width: '100%',
				height: '100%',
				style: 'background-size:100% 250px',
				html: '<div id="bigbox-ad' + sid + '"></div>'
			},{
                docked: 'bottom',
                xtype: 'panel',
                style: 'background-color: #202223;',
                html: '<div class="swipe-continue">Swipe to continue &gt;&gt;</div>'
            }],
			listeners: {
            	dragstart: {
            		element: 'element',
            		fn: function() {
                        Pheme.helpers.DfpAds.removeAds();
            		}
              	}
			}
		});
	},
    /**
     * Creates custom Carousel
     * @param  {int} internalId Carousel Identifier
     * @return {obj}            Returns the newly created Carousel object
     */
    customCarouselFactory: function(internalId) {
        // PANEL SNAPPING
        var customCarousel = Ext.create('Pheme.view.CustomCarousel', {
            direction: 'vertical',
            directionLock: true,
            cls: 'story-wrapper',
            fullscreen: true
        });

        // CONTINUOUS SCROLLING
        // var customCarousel = Ext.create('Ext.Container', {
        //     fullscreen: true,
        //     cls: 'story-wrapper',
        //     scrollable: {
        //         direction: 'vertical',
        //         directionLock: true,
        //         momentumEasing: {
        //             momentum: {
        //                 acceleration: 10,
        //                 friction: 1
        //             },
        //             bounce: {
        //                 acceleration: 0.0001,
        //                 springTension: 1
        //             }
        //         },
        //         outOfBoundRestrictFactor: 0   
        //     }
        // });

        if(internalId) {
            customCarousel.internalId = internalId;
        }

        return customCarousel;
    },
    /**
     * Creates a carousel panel
     * @return {obj} Returns a basic panel
     */
    carouselPanelFactory: function() {
        return Ext.create('Ext.Panel', {
            layout: 'vbox'
        });
    },
    /**
     * Creates an image panel
     * @param  {obj}  story    The Story object of the desired Story.
     * @param  {Boolean} isRetina Is it a Retina device? True/False
     * @return {obj}           Returns an image panel
     */
    imagePanelFactory: function(story, isRetina) {
        // PANEL SNAPPING
        return Ext.create('Ext.Img', {
            flex: 1,
            cls: 'story-feature-image',
            src: (isRetina) ? story.data.featured_image_url_x2 : story.data.featured_image_url
        });

        // CONTINUOUS SCROLLING
        // return Ext.create('Ext.Panel', {
        //     cls: 'story-feature-image',
        //     html: (function(story) {
        //         if(Pheme.app.isRetina()) {
        //             return '<img class="show-feature" src="' + story.data.featured_image_url_x2 + '" width="640" height="384" />';
        //         }
        //         else {
        //             return '<img class="show-feature" src="' + story.data.featured_image_url + '" width="320" height="192" />';
        //         }
        //     })(story)
        // });
    },
    /**
     * [imageCaptionPanelFactory description]
     * @param  {[type]} story [description]
     * @return {[type]}       [description]
     */
    imageCaptionPanelFactory: function(story) {
        return Ext.create('Ext.Panel', {cls: 'credit', html: story.data.credit});
    },
    /**
     * [generateContentPanel description]
     * @param  {[type]} story         [description]
     * @param  {[type]} carouselPanel [description]
     * @return {[type]}               [description]
     */
    generateContentPanel: function(story, carouselPanel) {
        var genericStoryController = this;

        return Ext.create('Ext.Panel', {
            docked: 'bottom',
            flex: 1,
            items: [
                {
                    xtype: 'panel',
                    cls: 'story-headline',
                    html: story.data.title
                },
                {
                    xtype: 'panel',
                    cls: 'timestamp-wrappper',
                    items: [
                        {
                            xtype: 'panel',
                            itemId: 'storyTimestamp',
                            cls: 'story-timestamp left',
                            //Need to do this to account for when the Updated label is added. If the Updated property is enabled in the story from Wordpress, then
                            //we should add an element with the Updated text.
                            html: (function(story) {
                                var returnString = '<div class="timestamp left">';
                                if(story.data.updated) {
                                    returnString += '<span class="updated">Updated </span>';
                                }
                                returnString += Ext.util.Format.date(story.data.date_utc, "M j, Y g:i A") + '</div>';
                                return returnString;
                            })(story, carouselPanel)

                        },
                        {
                            xtype: 'panel',
                            cls: 'right inner-follow-button inner-follow',
                            itemId: 'innerStoryFollowButton',
                            listeners: {
                                tap: {
                                    element: 'element',
                                    fn: function(self) {
                                        var followed = Pheme.helpers.FavoriteStories.toggleFavoriteStory(story.data.id);

                                        var followButton = genericStoryController.getFollowButtonElement();

                                        if(followed === 1) {
                                            this.addCls('inner-following');

                                            if(followButton) {
                                                followButton.setSrc('resources/images/toolbar/follow-active-btn-wh.png');
                                            }
                                        } 
                                        else {
                                            this.removeCls('inner-following');

                                            if(followButton) {
                                                followButton.setSrc('resources/images/toolbar/follow-btn-wh.png');
                                            }
                                        }
                                    }
                                }
                            }
                        },
                        {
                            xtype: 'panel',
                            cls: 'clear'
                        }
                    ]
                },
                {
                    xtype: 'panel',
                    cls: 'story-content story-body',
                    html: story.data.content[0]
                }
            ]
        });
    },
    /**
     * [addContentPanelsToCarousel description]
     * @param {[type]} story         [description]
     * @param {[type]} carouselPanel [description]
     * @param {[type]} contentPanel  [description]
     */
    addContentPanelsToCarousel: function(story, carouselPanel, contentPanel) {
        carouselPanel.setItems([
        {
            xtype: 'panel',
            docked: 'bottom',
            html: '<span class="paging-length">1/' + story.data.content.length + '</span>',
            cls: 'pager'
        }, 
        contentPanel]);

        return carouselPanel;
    },
    /**
     * [checkIfFeaturedStoryExists description]
     * @param  {[type]} story [description]
     * @return {[type]}       [description]
     */
    checkIfFeaturedStoryExists: function(story) {

        if(story.data.featured_image_url && story.data.featured_image_url.length > 0) {
            return true;
        } else {
            return false;
        }
    },
    /**
     * [checkIfGallery description]
     * @param  {[type]} story [description]
     * @return {[type]}       [description]
     */
    checkIfGallery: function(story) {

        if('gallery' === story.data.post_format) {
            return true;
        } else {
            return false;
        }
    },
    /**
     * [checkIfCreditExists description]
     * @param  {[type]} story [description]
     * @return {[type]}       [description]
     */
    checkIfCreditExists: function(story) {

        if(story.data.credit) {
            return true;
        } else {
            return false;
        }
    },
    /**
     * [generateStoryContentPanel description]
     * @param  {[type]}  storyContent [description]
     * @param  {[type]}  pageIndex    [description]
     * @param  {Boolean} isLast       [description]
     * @return {[type]}               [description]
     */
    generateStoryContentPanel: function(storyContent, pageIndex, isLast) {
        var genericStoryController = this;
        if(isLast === undefined) isLast = false;

        return Ext.create('Ext.Panel', {
            cls: (isLast) ? 'story-wrapper last' : 'story-wrapper',
            fullscreen: true,
            items: [
                {
                    xtype: 'panel',
                    height: '100%',
                    html: '<div class="story-content-page2 story-body">' + storyContent[pageIndex] + '</div>'
                }, 
                {
                    xtype: 'panel',
                    docked: 'bottom',
                    html: '<span class="paging-length">' + (pageIndex + 1) + '/' + storyContent.length + '</span>',
                    cls: 'pager'
                }
            ],
            listeners: {
                painted: function(panel){
                    if (genericStoryController.checkIfOSisIOS()){
                        if(genericStoryController.checkForIFrames(panel)) {
                            panel.hide();
                            panel.show();
                        }
                    }
                }
            }
        })
    },
    /**
     * [addStoryContentPanelsToCarousel description]
     * @param {[type]} carousel     [description]
     * @param {[type]} storyContent [description]
     */
    addStoryContentPanelsToCarousel: function(carousel, storyContent) {
        for (var i = 1; i < storyContent.length; i++) {
            var isLast = (i == storyContent.length - 1);
            carousel.add(this.generateStoryContentPanel(storyContent, i, isLast));
        }
    },
    /**
     * [setCarouselProperties description]
     * @param {[type]} carousel [description]
     * @param {[type]} story    [description]
     */
    setCarouselProperties: function(carousel, story) {
        carousel.currentStoryId = story.data.id;
        carousel.currentStoryTitle = story.data.title;
        carousel.currentStoryDate = Ext.util.Format.date(story.data.date_utc, "M j, Y g:i A");
        carousel.shareMessage = story.data.title;
        carousel.shareImage = story.data.featured_image_url;
        carousel.shareLink = story.data.permalink;
    },
    /**
     * [checkIfOSisIOS description]
     * @return {[type]} [description]
     */
    checkIfOSisIOS: function() {
        if(Ext.os.is('iOS')) {
            return true;
        } else {
            return false;
        }
    },
    /**
     * [checkForIFrames description]
     * @param  {[type]} contentPanel [description]
     * @return {[type]}              [description]
     */
    checkForIFrames: function(contentPanel) {
        var iframes = contentPanel.dom.querySelectorAll(".embed, .embed-youtube, .embed-vimeo, iframe[src*='youtube.com'], iframe[src*='youtu.be'], iframe[src*='vimeo.com'], iframe[src*='vine.co'], iframe[src*='soundcloud.com']");
        if(iframes && iframes.length > 0) {
            return true;
        } else {
            return false;
        }
    },
    /**
     * [toggleFollowStory description]
     * @param  {[type]} storyId [description]
     * @return {[type]}         [description]
     */
    toggleFollowStory: function(storyId) {
        return Pheme.helpers.FavoriteStories.toggleFavoriteStory(storyId);
    },
    /**
     * [getFollowButtonElement description]
     * @return {[type]} [description]
     */
    getFollowButtonElement: function() {
        var parent = Ext.Viewport.getActiveItem().id;
        return Ext.ComponentQuery.query('#' + parent + ' #btn_follow')[0];
    },
    /**
     * [toggleFollowingCSSClass description]
     * @param  {[type]}  element     [description]
     * @param  {Boolean} isFollowing [description]
     * @return {[type]}              [description]
     */
    toggleFollowingCSSClass: function(element, isFollowing) {
        if(isFollowing) {
            element.addCls('inner-following');
        } else {
            element.removeCls('inner-following');
        }
    }

});
