/**
 *  @class Pheme.controller.Story
 *  @extends Pheme.controller.GenericStory
 *  Description: Controller for the default Story view
 */
Ext.define('Pheme.controller.Story', {
    extend: 'Pheme.controller.GenericStory',
    requires: ['Pheme.view.CustomCarousel'],
    adSlot: null,
    firstView: true,
    config: {
        refs: {
            toolbar: 'storytitlebar',
            storyCarousel: '#StoryView',
            storyAd: '#storyviewAd',
            followButton : "#btn_follow"
        },
        control: {
            storyCarousel: {
                /**
                 * [description]
                 * @param  {[type]} e [description]
                 * @return {[type]}   [description]
                 */
                'story.link.tap': function(e) {
                    e.target.onclick = function() { return false };
                    e.stopEvent();
                    e.preventDefault();
                    e.stopPropagation();
                    
                    Pheme.helpers.Common.linktapped(e.target, Ext.Viewport.getActiveItem().getActiveItem().internalId);
                },
                /**
                 * [activate description]
                 * @param  {[type]} storyCarousel [description]
                 * @param  {[type]} viewport      [description]
                 * @param  {[type]} previousview  [description]
                 * @param  {[type]} eOpts         [description]
                 * @return {[type]}               [description]
                 */
                activate: function(storyCarousel, viewport, previousview, eOpts) {
                    this.adInserted = false;
                    this.numberOfSwipes = 0;

                    //We have a var for the current store which is for accessing Search and Followed Stories stores. If it hasn't been set, it should return undefined, so we account for this by defaulting to Stories store.
                    var store = Ext.data.StoreManager.get(Pheme.app.currentStoryStore) || Ext.data.StoreManager.get('Stories'), 
                        storyId = Pheme.app.currentStoryId,
                        retrievedStory = (storyId) ? store.getById(storyId) : store.getAt(0); //For retrieved story, we have a var which keeps track of the current story, if it's at 0 it means that we are swiping so we should just start from the beginning.

                    this.currentStoryId = retrievedStory.data.id;
                    this.currentStoryTitle = retrievedStory.data.title;
                    this.currentStoryDate = Ext.util.Format.date(retrievedStory.data.date_utc, "M j, Y g:i A");

                    if(store.indexOf(retrievedStory) >= store.getAllCount() - 2) {
                        this.loadNextPageOfDataIntoStore(store);
                    }

                    this.setupStoriesForCarouselActivation(retrievedStory, storyCarousel, store);

                    // reset firstView - Each time the carousel is presented to the user when activated
                    this.firstView = true;

                    if(this.shouldLoadInterstitialIntoStoryCarousel(storyId, store) && !this.adInserted) {
                        if(this.isAtEndOfCarousel(retrievedStory.internalId, store)) {
                            this.loadInterstitialAdIntoStoryCarousel(storyCarousel, 'back', 2);
                        } else if(this.isAtBeginningOfCarousel(retrievedStory.internalId, store)) {
                            this.loadInterstitialAdIntoStoryCarousel(storyCarousel, 'forward', 2);
                        } else {
                            this.loadInterstitialAdIntoStoryCarousel(storyCarousel, 'forward', 3);
                        }

                        this.adInserted = true;
                    } else {
                        this.adInserted = false;
                    }
                },
                /**
                 * [show description]
                 * @param  {[type]} carousel [description]
                 * @param  {[type]} eOpts    [description]
                 * @return {[type]}          [description]
                 */
                show: function(carousel, eOpts) {
                    var store = Ext.data.StoreManager.get(Pheme.app.currentStoryStore) || Ext.data.StoreManager.get('Stories');

                    if(this.isAtBeginningOfCarousel(Pheme.app.currentStoryId, store)) {
                        carousel.setActiveItem(0);
                    } else if(this.isAtEndOfCarousel(Pheme.app.currentStoryId, store)) {
                        carousel.setActiveItem(2);
                    } else {
                        carousel.setActiveItem(1);
                    }

                    Pheme.helpers.Common.updateReadStory(carousel.getActiveItem().currentStoryId);
                },
                /**
                 * [hide description]
                 * @param  {[type]} carousel [description]
                 * @param  {[type]} eOpts    [description]
                 * @return {[type]}          [description]
                 */
                hide: function(carousel, eOpts) {
                    carousel.removeAll(false, false);
                },
                /**
                 * [activeitemchange description]
                 * @param  {[type]} carousel      [description]
                 * @param  {[type]} newActiveItem [description]
                 * @param  {[type]} oldActiveItem [description]
                 * @param  {[type]} eOpts         [description]
                 * @return {[type]}               [description]
                 */
                activeitemchange: function(carousel, newActiveItem, oldActiveItem, eOpts) {
                    var storyId = newActiveItem.internalId,
                        store = Ext.data.StoreManager.get(Pheme.app.currentStoryStore),
                        retrievedStory = store.getById(storyId),
                        toolbar = this.getToolbar();

                    //Force to clear selected star on active item change to kick in early.
                    this.getFollowButton().setSrc('resources/images/toolbar/follow-btn-wh.png');
            
                    this.trackUserMetrics(storyId, retrievedStory);

                    if(newActiveItem.aliases === 'adPanel') {
                        this.hideToolbar(toolbar);
                        Pheme.helpers.DfpAds.createDfpAd(this.generateOptionsForInterstitial(this.getStoryDataForInterstitialAd(carousel, store)), true);
                    } 
                    else {
                        carousel.fireEvent('toggleFollowButton', storyId, 'storyViewTitleBar', newActiveItem.getItemId());
                        
                        this.showToolbar(toolbar);
                        
                        Pheme.helpers.DfpAds.renderBannerAd(retrievedStory, '#252525', true, false);
                    }

                    if(this.isAtBeginningOfCarousel(storyId, store)) {
                        this.resetSwipeCount();
                    }

                    var storyInterstitialOptions = {
                        carousel: carousel,
                        store: store,
                        storyId: storyId
                    };

                    if(this.isGoingForwardThroughCarousel(carousel.animationDirection)) {

                        this.numberOfSwipes++;

                        if(carousel.getActiveIndex() > store.getAllCount() - 8) {

                            this.loadNextPageOfDataIntoStore(store);
                        }

                        storyInterstitialOptions.direction = 'forward';

                        this.insertInterstitialAdIntoStoryCarousel(storyInterstitialOptions);

                        var nextStory = this.getNextStoryId(storyId, store);
                        this.insertStoryIntoCarousel(nextStory, carousel, store);

                    } else if(false === this.isGoingForwardThroughCarousel(carousel.animationDirection)) {

                        this.numberOfSwipes++;

                        storyInterstitialOptions.direction = 'back';

                        this.insertInterstitialAdIntoStoryCarousel(storyInterstitialOptions);

                        var prevStory = this.getPrevStoryId(storyId, store);
                        this.insertStoryIntoCarouselAt(0, prevStory, carousel, store);
                    }

                    //Force the frame to reload to stop play any audio/video when moved on to next story.
                    this.forceFrameReloadForAndroid(oldActiveItem);
                    this.firstView = false;
                }
            }
        }
    },
    /**
     * [setupStoriesForCarouselActivation description]
     * @param  {[type]} retrievedStory [description]
     * @param  {[type]} storyCarousel  [description]
     * @param  {[type]} store          [description]
     * @return {[type]}                [description]
     */
    setupStoriesForCarouselActivation: function(retrievedStory, storyCarousel, store) {
        var nextStory = this.getNextStoryId(retrievedStory.internalId, store);
        var prevStory = this.getPrevStoryId(retrievedStory.internalId, store);

        this.insertStoryIntoCarousel(prevStory, storyCarousel, store);
        this.insertStoryIntoCarousel(retrievedStory.internalId, storyCarousel, store);
        this.insertStoryIntoCarousel(nextStory, storyCarousel, store);
    },
    /**
     * [isAtBeginningOfCarousel description]
     * @param  {[type]}  storyId [description]
     * @param  {[type]}  store   [description]
     * @return {Boolean}         [description]
     */
    isAtBeginningOfCarousel: function(storyId, store) {
        if(this.getPrevStoryId(storyId, store) === storyId) {
            return true;
        } else {
            return false;
        }
    },
    /**
     * [isAtEndOfCarousel description]
     * @param  {[type]}  storyId [description]
     * @param  {[type]}  store   [description]
     * @return {Boolean}         [description]
     */
    isAtEndOfCarousel: function(storyId, store) {
        if(this.getNextStoryId(storyId, store) === storyId) {
            return true;
        } else {
            return false;
        }
    },
    /**
     * [trackUserMetrics description]
     * @param  {[type]} storyId [description]
     * @param  {[type]} story   [description]
     * @return {[type]}         [description]
     */
    trackUserMetrics: function(storyId, story) {
        if (storyId) {
            this.trackUserInteractions(story);
            this.trackUserNavigationIfFirstView(this.firstView, story);
        }
    },
    /**
     * [trackUserInteractions description]
     * @param  {[type]} story [description]
     * @return {[type]}       [description]
     */
    trackUserInteractions: function(story) {
        Pheme.helpers.Omniture.pageView("story", story);
    },
    /**
     * [trackUserNavigationIfFirstView description]
     * @param  {[type]} firstView [description]
     * @param  {[type]} story     [description]
     * @return {[type]}           [description]
     */
    trackUserNavigationIfFirstView: function(firstView, story) {
        if(!firstView) {
            Pheme.helpers.Omniture.storyAction("Navigation - Swipe", story);

            if(null !== story.internalId) {
                Pheme.helpers.Common.updateReadStory(story.internalId);
            }
        }
    },
    /**
     * [isGoingForwardThroughCarousel description]
     * @param  {[type]}  direction [description]
     * @return {Boolean}           [description]
     */
    isGoingForwardThroughCarousel: function(direction) {
        if(-1 === direction) {
            return true;
        } else if(1 === direction) {
            return false;
        } else {
            return null;
        }
    },
    /**
     * [generateOptionsForInterstitial description]
     * @param  {[type]} storyForIntersitial [description]
     * @return {[type]}                     [description]
     */
    generateOptionsForInterstitial: function(storyForIntersitial) {

        if(!storyForIntersitial) {
            return;
        }

        //From here we just take the ad code values in the story's data and pass that along to create the ad.
        var options = {
            'adSize': 'BIGBOX',
            'tags': {
                'pr': Pheme.Config.fetchCurrentMarket().adCode,
                'loc': storyForIntersitial.data.ad_codes.loc,
                'imp': storyForIntersitial.data.ad_codes.imp,
                'page': storyForIntersitial.data.ad_codes.page,
                'ck': storyForIntersitial.data.ad_codes.ck,
                'nk': storyForIntersitial.data.ad_codes.nk
            },
            'backgroundColor': '#252525'
        };

        options['adUnitId'] = (storyForIntersitial.data.ad_codes.ad) ? '/3081/' + Pheme.Config.fetchCurrentMarket().adCode + '.' + (Ext.os.is.iOS ? 'ip' : 'aroid') + '/' + storyForIntersitial.data.ad_codes.ad : 'news';

        if(Pheme.helpers.DfpAds.getTestAds()) {
            options['tags']['test'] = 'yes';
        }

        //This is for sub-category keys in ads
        if(storyForIntersitial.data.ad_codes.sck && storyForIntersitial.data.ad_codes.sck.length > 0) {
            options['tags']['sck'] = storyForIntersitial.data.ad_codes.sck;
        }

        return options;
    },
    /**
     * [loadNextPageOfDataIntoStore description]
     * @param  {[type]} carousel [description]
     * @param  {[type]} store    [description]
     * @return {[type]}          [description]
     */
    loadNextPageOfDataIntoStore: function(store) {
        store.nextPage({ addRecords: true });
    },
    /**
     * [getNextStoryId description]
     * @param  {[type]} storyId [description]
     * @param  {[type]} store   [description]
     * @return {[type]}         [description]
     */
    getNextStoryId: function(storyId, store) {
        var currentStoryIndex = store.find('id', storyId);
        var nextStory = store.getAt(currentStoryIndex + 1);

        try {
            if(this.isStoryANativeAd(nextStory)) {
                nextStory = store.getAt(currentStoryIndex + 2);
            }
            
            return nextStory.internalId;
        } catch(err) {
            return storyId;
        }

    },
    /**
     * [getPrevStoryId description]
     * @param  {[type]} storyId [description]
     * @param  {[type]} store   [description]
     * @return {[type]}         [description]
     */
    getPrevStoryId: function(storyId, store) {
        var currentStoryIndex = store.find('id', storyId);
        var prevStory = store.getAt(currentStoryIndex - 1);

        try {
            if(this.isStoryANativeAd(prevStory)) {
                prevStory = store.getAt(currentStoryIndex - 2);
            }
            
            return prevStory.internalId;
        } catch(err) {
            return storyId;
        }
    },
    /**
     * [insertStoryIntoCarousel description]
     * @param  {[type]} storyId  [description]
     * @param  {[type]} carousel [description]
     * @param  {[type]} store    [description]
     * @return {[type]}          [description]
     */
    insertStoryIntoCarousel: function(storyId, carousel, store) {

        var storyIndex = store.find('id', storyId);
        var story = store.getAt(storyIndex);

        if(this.isStoryANativeAd(story)) {
            story = store.getAt(++storyIndex);
        }

        if(story && story.hasOwnProperty('data')) {
            if(this.storyIsNotInCarousel(story.internalId)) {
                carousel.add(this.addStory(story));
            } else {
                return false;
            }
        } else {
            return null;
        }
    },
    /**
     * [insertStoryIntoCarouselAt description]
     * @param  {[type]} index    [description]
     * @param  {[type]} storyId  [description]
     * @param  {[type]} carousel [description]
     * @param  {[type]} store    [description]
     * @return {[type]}          [description]
     */
    insertStoryIntoCarouselAt: function(index, storyId, carousel, store) {
        var storyIndex = store.find('id', storyId);
        var story = store.getAt(storyIndex);

        if(story && story.hasOwnProperty('data')) {
            if(this.storyIsNotInCarousel(story.internalId)) {
                carousel.insert(index, this.addStory(story));
            }
        }
    },
    /**
     * [storyIsNotInCarousel description]
     * @param  {[type]} storyId [description]
     * @return {[type]}         [description]
     */
    storyIsNotInCarousel: function(storyId) {
        var child = Ext.ComponentQuery.query('carousel > container[currentStoryId=' + storyId + ']');

        if(child.length === 0) {
            return true;
        } else {
            return false;
        }
    },
    /**
     * [shouldLoadInterstitialIntoStoryCarousel description]
     * @param  {[type]} storyId [description]
     * @param  {[type]} store   [description]
     * @return {[type]}         [description]
     */
    shouldLoadInterstitialIntoStoryCarousel: function(storyId, store) {

        if(this.firstView && this.numberOfSwipes === 0) {
            return true;
        }

        if(this.numberOfSwipes === 4 && !this.isAtBeginningOfCarousel(storyId, store)) {
            return true;
        } else {
            return false;
        }
    },
    /**
     * [insertInterstitialAdIntoStoryCarousel description]
     * @param  {[type]} options [description]
     * @return {[type]}         [description]
     */
    insertInterstitialAdIntoStoryCarousel: function(options) {
        if(this.shouldRemoveInterstitialFromCarousel()) {
            this.removeInterstitialAdFromCarousel(options.carousel, options.direction);
        }

        if(this.shouldLoadInterstitialIntoStoryCarousel(options.storyId, options.store) && !this.adInserted) {
            this.loadInterstitialAdIntoStoryCarousel(options.carousel, options.direction);
            this.adInserted = true;
        } else {
            this.adInserted = false;
        }
    },
    /**
     * [loadInterstitialAdIntoStoryCarousel description]
     * @param  {[type]} carousel  [description]
     * @param  {[type]} direction [description]
     * @param  {[type]} index     [description]
     * @return {[type]}           [description]
     */
    loadInterstitialAdIntoStoryCarousel: function(carousel, direction, index) {

        this.resetSwipeCount();

        var currentIndex = carousel.getActiveIndex(),
            adInsertIndex;

        if(direction === 'forward') {
            adInsertIndex = currentIndex + 2;

        } else {
            adInsertIndex = currentIndex;
        }

        if(index) {
            adInsertIndex = index;
        }

        var currentStoryId = carousel.getActiveItem().currentStoryId;
        var newInterstitialAd = this.getAd(currentStoryId);

        newInterstitialAd.internalId = currentStoryId;

        carousel.insert(adInsertIndex, newInterstitialAd);
    },
    /**
     * [removeInterstitialAdFromCarousel description]
     * @param  {[type]} carousel [description]
     * @return {[type]}          [description]
     */
    removeInterstitialAdFromCarousel: function(carousel) {
        var interstitial = Ext.ComponentQuery.query('panel[aliases="adPanel"]');
        carousel.remove(interstitial[0], true);
    },
    /**
     * [shouldRemoveInterstitialFromCarousel description]
     * @return {[type]} [description]
     */
    shouldRemoveInterstitialFromCarousel: function() {
        if(this.numberOfSwipes === 2 && !this.firstView) {
            return true;
        } else {
            return false;
        }
    },
    /**
     * [resetSwipeCount description]
     * @return {[type]} [description]
     */
    resetSwipeCount: function() {
        this.numberOfSwipes = 0;
    },
    /**
     * [getStoryDataForInterstitialAd description]
     * @param  {[type]} carousel [description]
     * @param  {[type]} store    [description]
     * @return {[type]}          [description]
     */
    getStoryDataForInterstitialAd: function(carousel, store) {
        //For Interstitial Ads, rules for targeting are as follows:
        //Given the example: story1=sports > interstitial > story2=life
        //a. when swiping forwards, the bigbox/interstitial's category would be "life"
        //b. when swiping backwards, the bigbox/interstitial's category would be "sports"

        //In order to satisfy the rules above, we need to use the internal id of the preceeding or the
        //succeeding story.
        var storyForIntersitial;

        if(this.isGoingForwardThroughCarousel(carousel.animationDirection)) {
            storyForIntersitial = store.getById(this.getNextStoryInCarouselForInterstitial(carousel));
        } else {
            storyForIntersitial = store.getById(this.getPreviousStoryInCarouselForInterstitial(carousel));
        }

        return storyForIntersitial;
    },
    /**
     * [getNextStoryInCarouselForInterstitial description]
     * @param  {[type]} carousel [description]
     * @return {[type]}          [description]
     */
    getNextStoryInCarouselForInterstitial: function(carousel) {
        var nextStoryIndex = carousel.getActiveIndex(),
            storyForInterstitialId,
            doubleStoryIndex = nextStoryIndex + 2;

        if(undefined === carousel.getAt(doubleStoryIndex) || this.isStoryANativeAd(carousel.getAt(doubleStoryIndex))) {
            storyForInterstitialId = carousel.getAt(nextStoryIndex).internalId;
        } else {
            storyForInterstitialId = carousel.getAt(doubleStoryIndex).internalId;
        }

        return storyForInterstitialId;
    },
    /**
     * [getPreviousStoryInCarouselForInterstitial description]
     * @param  {[type]} carousel [description]
     * @return {[type]}          [description]
     */
    getPreviousStoryInCarouselForInterstitial: function(carousel) {
        var prevStoryIndex = carousel.getActiveIndex(),
            storyForInterstitialId;

        //if we are at the beginning, we can just specify that we want the first story's id.
        if(prevStoryIndex === 0) {
            storyForInterstitialId = carousel.getAt(1).internalId;
        } else { //Since we are not at the beginning, we can just use the active index of the carousel.
            storyForInterstitialId = carousel.getAt(prevStoryIndex).internalId;
        }

        return storyForInterstitialId;
    },
    isStoryANativeAd: function(story) {
        if(typeof story != 'undefined' && typeof story.isNativeAd != 'undefined' && story.isNativeAd) {
            return true;
        } else {
            return false;
        }
    },
    /**
     * [showToolbar description]
     * @param  {[type]} toolbar [description]
     * @return {[type]}         [description]
     */
    showToolbar: function(toolbar) {
        toolbar.removeCls('toolbarhide');
    },
    /**
     * [hideToolbar description]
     * @param  {[type]} toolbar [description]
     * @return {[type]}         [description]
     */
    hideToolbar: function(toolbar) {
        toolbar.addCls('toolbarhide');
    },
    /**
     * [forceFrameReloadForAndroid description]
     * @param  {[type]} oldActiveItem [description]
     * @return {[type]}               [description]
     */
    forceFrameReloadForAndroid: function(oldActiveItem) {
        try{
            if (Ext.os.is('Android')) {
                var iframes = oldActiveItem.body.dom.querySelectorAll(".embed-youtube, iframe[src*='vimeo.com'],  iframe[src*='vine.co'], iframe[src*='soundcloud.com']");
                for(var i = 0; i < iframes.length; i++)
                    iframes[i].src = iframes[i].src;
            }
        }catch(e){console.log(e);}
    },
    /**
     * [constructor description]
     * @param  {[type]} config [description]
     * @return {[type]}        [description]
     */
    constructor: function(config) {
        this.initConfig(config);
        this.eventsQueue.push({ caller: 'StoryView', eventTrigger: 'activeitemchange', functionToRun: Pheme.helpers.ControllerEvents.sendAnalytics, args: [] });
        this.eventsQueue.push({ caller: 'StoryView', eventTrigger: 'activeitemchange', functionToRun: Pheme.helpers.ControllerEvents.renderAd, args: [] });
    },
    /**
     * [init description]
     * @return {[type]} [description]
     */
    init: function() {
        this.callParent(arguments);
    }
});
