/**
 *  @class Pheme.controller.StoryTitleBar
 *  @extends Ext.app.Controller
 *  Description: Controller for the Title Bar functionality
 */
Ext.define('Pheme.controller.StoryTitleBar', {
    extend: 'Ext.app.Controller',
    config: {
    	refs: {
		    backButton: 'component[itemId=btn_back]',
		    shareButton: 'component[itemId=btn_share]',
		    followButton: 'component[itemId=btn_follow]',
		    homeButton: 'component[itemId=nav_icon]',
	        alertButton: 'component[itemId=btn_alert]',
			alertModalView: '#FavoriteView',
			storyCarouselView: '#StoryView',
			alertStoryCarouselView: '#AlertStoryView',
			searchStoryCarouselView: '#SearchStoryView',
			main: 'Main',
			updatedFollowCount: '#followingCountTitleBarStory'
		},
		control: {
			updatedFollowCount: {
				/**
				 * [initialize description]
				 * @param  {[type]} label [description]
				 * @param  {[type]} eOpts [description]
				 * @return {[type]}       [description]
				 */
				initialize: function(label, eOpts) {
					label.element.on('tap', function(b, e) {
						Pheme.helpers.FavoriteStories.showFavoriteView(); 
					});
				}
			},
	    	backButton: {
	    		/**
	    		 * [tap description]
	    		 * @return {[type]} [description]
	    		 */
	    		tap: function() {
		    		var storyCarousel = Ext.Viewport.getActiveItem();
	    			if(storyCarousel){
	    				if(storyCarousel.id === 'StoryView' || storyCarousel.id === 'AlertStoryView') {
    						var storyItem = storyCarousel.innerItems[storyCarousel.getActiveIndex()];
			    			//TODO: Need to move away from using Pheme.app.currentStoryStore. Needs refactoring.
			    			Pheme.app.currentStoryStore = 'Stories';

			        		this.backToView('#mainViewport');

			        		var storiesList = Ext.getCmp('storiesList');
			        		storiesList.getScrollable().getScroller().setDisabled(false);
			        		
			    		} else if(storyCarousel.id === 'SearchStoryView') 
			    			this.backToView('#SearchView');

			    		Pheme.helpers.Omniture.storyAction("Back Icon Click", storyItem.internalId);
			    	}
				}
	    	},
	    	shareButton: {
				tap: 'shareStory'
	    	},
	    	followButton: {
	    		/**
	    		 * [tap description]
	    		 * @return {[type]} [description]
	    		 */
	    		tap: function() {
	    			var storyCarousel = Ext.Viewport.getActiveItem();
    				var storyItem = storyCarousel.innerItems[storyCarousel.getActiveIndex()];

	    			var result = Pheme.helpers.FavoriteStories.toggleFavoriteStory(storyItem.internalId);

	    			if(result == 1) {
	    				this.showFollowStoryBalloon();
	    			}

	    			this.toggleFollowButton(storyItem.internalId, Ext.Viewport.getActiveItem().id, Ext.Viewport.getActiveItem().getActiveItem().getItemId());
	    		}
			},
	    	homeButton: {
	    		/**
	    		 * [tap description]
	    		 * @return {[type]} [description]
	    		 */
	    		tap: function() {
	    			var storyCarousel = Ext.Viewport.getActiveItem();
	    			var storyItem = storyCarousel.innerItems[storyCarousel.getActiveIndex()];
    				Pheme.helpers.Omniture.storyAction("Home Icon Click", storyItem.internalId);

	    			var store = Ext.getStore('Stories'),
	    				filterBar = Ext.getCmp('filter_bar');

	    			store.getProxy().setUrl(Pheme.Config.fetchCurrentMarket().endpoint + '/stories'),
	 	        	store.loadPage(1),

		 			//TODO: Need to move away from using Pheme.app.currentStoryStore. Needs refactoring.
		    		Pheme.app.currentStoryStore = 'Stories';
	 	        	
	 	        	if(filterBar && !filterBar.isHidden()) {
						filterBar.hide();
					}
					
					this.backToView('#mainViewport');
					storiesList = Ext.getCmp('storiesList');
					storiesList.getScrollable().getScroller().setDisabled(false);
		 			storiesList.getScrollable().getScroller().scrollTo(0, 0);
		 			storiesList.removeCls("filtered");
	    		}
	    	},
			alertButton: {
				/**
				 * [tap description]
				 * @param  {[type]} ) {            Pheme.helpers.FavoriteStories.showFavoriteView( [description]
				 * @return {[type]}   [description]
				 */
				tap: function() { Pheme.helpers.FavoriteStories.showFavoriteView(); }
			},
			storyCarouselView: {
				/**
				 * [description]
				 * @param  {[type]} storyId  [description]
				 * @param  {[type]} titleBar [description]
				 * @param  {[type]} viewId   [description]
				 * @return {[type]}          [description]
				 */
				'toggleFollowButton': function(storyId, titleBar, viewId) {
					this.toggleFollowButton(storyId, titleBar, viewId);
				}
			},
			alertStoryCarouselView: {
				/**
				 * [description]
				 * @param  {[type]} storyId  [description]
				 * @param  {[type]} titleBar [description]
				 * @param  {[type]} viewId   [description]
				 * @return {[type]}          [description]
				 */
				'toggleFollowButton': function(storyId, titleBar, viewId) {
					this.toggleFollowButton(storyId, titleBar, viewId);
				}
			},
			searchStoryCarouselView: {
				/**
				 * [description]
				 * @param  {[type]} storyId  [description]
				 * @param  {[type]} titleBar [description]
				 * @param  {[type]} viewId   [description]
				 * @return {[type]}          [description]
				 */
				'toggleFollowButton': function(storyId, titleBar, viewId) {
					this.toggleFollowButton(storyId, titleBar, viewId);
				}
			}
	    }
	},
	/**
	 * [backToView description]
	 * @param  {[type]} view [description]
	 * @return {[type]}      [description]
	 */
    backToView: function(view) {
    	Pheme.helpers.Common.switchElement(this.getMain().element, false, false);

		Pheme.helpers.Common.switchView(view, { direction: 'right' });
	},
    
    /**
     * [toMainView description]
     * @param  {[type]} view [description]
     * @return {[type]}      [description]
     */
    toMainView: function(view) {
        Pheme.helpers.Common.switchView(view, { direction: 'right' });
           
        try  {
            StatusBar.show();
            StatusBar.overlaysWebView(true);
        } 
        catch(e) {
           console.log(e);
        }
    },
           
	/**
	 * [shareStory description]
	 * @return {[type]} [description]
	 */
    shareStory: function() {
		var storyCarousel = Ext.Viewport.getActiveItem(),
	    	storyItem = storyCarousel.innerItems[storyCarousel.getActiveIndex()];
	    
	    //Removed image on share because Facebook was treating passed image as a post type of image rather than story.
	    var message = this.cleanString(storyItem.shareMessage);

		window.plugins.socialsharing.share(message, message, null, storyItem.shareLink);

    	Pheme.helpers.Omniture.storyAction("Social Media Share", storyItem.internalId);	
	},
	/**
	 * [cleanString description]
	 * @param  {[type]} str [description]
	 * @return {[type]}     [description]
	 */
    cleanString: function(str){
    	str = Ext.String.htmlDecode(str);
    	return str.replace("&#8217;", '’')
				.replace("&#8211;",'-')
				.replace("&#8212;", '—')
				.replace("&#8216;", '‘')
				.replace("&#8218;", '‚')
				.replace("&#8220;", '“')
				.replace("&#8221;", '”')
				.replace("&#8222;", '„')
				.replace("&#8482;", "™")
				.replace("&#160;", " ")
				.replace("&nbsp;", " ");
    },
    /**
     * [followStory description]
     * @param  {[type]} viewId [description]
     * @return {[type]}        [description]
     */
    followStory: function(viewId) {
    	var storyItem = Ext.Viewport.getActiveItem().getActiveItem();
    	var activeItemId = storyItem.getItemId();

		if(Pheme.helpers.FavoriteStories.toggleFavoriteStory(storyItem.currentStoryId)) {
			//Set tag for Urban Airship
			try {
				Pheme.helpers.UaAlerts.setPushInst(PushNotification);
				Pheme.helpers.UaAlerts.setTags(storyItem.currentStoryId.toString());
			} catch(e) {}
			this.showFollowStoryBalloon();
			Pheme.helpers.Omniture.storyAction("Follow Click", storyItem.currentStoryId);
    	} else {
    		//Remove tag for Urban Airship
    		try {
    			Pheme.helpers.UaAlerts.setPushInst(PushNotification);
				Pheme.helpers.UaAlerts.removeTag(storyItem.currentStoryId.toString());
			} catch(e) {}
			Pheme.helpers.Omniture.storyAction("Unfollow Click", storyItem.currentStoryId);
    	}
    	this.toggleFollowButton(storyItem.internalId, viewId, activeItemId);
    },
    /**
     * [showFollowStoryBalloon description]
     * @return {[type]} [description]
     */
    showFollowStoryBalloon: function() {
    	var balloon = Ext.getCmp('followStoryBalloon');

    	if(!balloon) {
    		balloon = Ext.create('Ext.Container', {
	    			cls: '',
	    			itemId: 'followStoryBalloon',
		    		html: '<p class="followStoryBalloon top">Added to Following</p>',
		    		hidden: true,
		    		right: 50,
		    		top: 50,
		    		height: 20
		    	});

    		Ext.Viewport.add(balloon);
    	}

		balloon.show();

		setTimeout(function() {
    		balloon.hide();
		}, '2000');
    },
    /**
     * [toggleFollowButton description]
     * @param  {[type]} storyId    [description]
     * @param  {[type]} titleBarId [description]
     * @param  {[type]} viewId     [description]
     * @return {[type]}            [description]
     */
	toggleFollowButton: function(storyId, titleBarId, viewId) {
		if(storyId === null || storyId === undefined) 
			return false;
		
		var titleBarFollowButton = Ext.ComponentQuery.query("#" + titleBarId + " #btn_follow" )[0];
		var innerStoryFollowButton = Ext.ComponentQuery.query("#" + viewId + " #innerStoryFollowButton" )[0];

		var followed = Pheme.helpers.FavoriteStories.isFavoriteStory(storyId);

		if(followed) {
			titleBarFollowButton.setSrc('resources/images/toolbar/follow-active-btn-wh.png');
			innerStoryFollowButton.addCls('inner-following');
		} else {
			titleBarFollowButton.setSrc('resources/images/toolbar/follow-btn-wh.png');
			innerStoryFollowButton.removeCls('inner-following');
		}
	}
});
