/**
 *  Common
 *  @class Pheme.helpers.Common
 *  Description: ...
 */
Ext.define('Pheme.helpers.Common', {
    singleton: true,
    config: {
    },
    /**
     * [diff description]
     * @param  {[type]} arrayA [description]
     * @param  {[type]} arrayB [description]
     * @return {[type]}        [description]
     */
    diff: function(arrayA, arrayB) {
        return arrayA.filter(function(i) { return arrayB.indexOf(i) < 0; } );
    },

    /**
     * Handle a link tapped within a story
     *
     * @param target
     * @param source - Story object or story id where link was tapped
     */
    linktapped: function(target, source){
        var link = (typeof target.dataset.href != 'undefined') ? target.dataset.href : target.href;
        
        //When link text is wrapped with elements such as strong, b, i etc, we need to find the link url from parent tags.
        if(link == null && typeof target.parentNode.href != 'undefined')
            link = target.parentNode.href;
        if(link == null && typeof target.parentNode.parentNode.href != 'undefined')
            link = target.parentNode.parentNode.href;
        
        var openInApp = false;
        if(target.dataset.storyline) {
            var story = Ext.getStore('Stories').getById(target.dataset.storyline);

            if(story) {
                this.showSingleStoryView(target.dataset.storyline, 'Stories');
            }
            else {
                var store = Ext.getStore('searchResult');
                store.getProxy().setUrl(Pheme.Config.fetchCurrentMarket().endpoint + '/stories?id=' + target.dataset.storyline);
                store.loadPage(1, {
                    scope: this,
                    callback: function(record, operation, success) {
                        if(success && store.getAllCount() > 0) {
                            this.showSingleStoryView(target.dataset.storyline, 'searchResult');
                        }
                        else
                            this.openInBrowser(link);   
                    }
                });
            }
        }
        else
            this.openInBrowser(link);

        Pheme.helpers.Omniture.storyLinkTap("Link Click", source, link);
    },
    /**
     * [openInBrowser description]
     * @param  {[type]} url [description]
     * @return {[type]}     [description]
     */
    openInBrowser: function(url) {
        if(url != null && typeof url != 'undefined')
            window.open(url, "_blank", "location=yes,toolbar=yes,transitionstyle=fliphorizontal,presentationstyle=fullscreen,enableViewportScale=yes");
    },
    /**
     * Open the view for a native ad
     * @param  {[type]} url [description]
     * @return {[type]}     [description]
     */
    presentNativeAdView: function(url) {
        if(url) {
            var nav = Ext.getCmp('NativeAdView');
            nav.setAdUrl(url);
            Pheme.helpers.Common.switchView('#NativeAdView');
        }
    },
    /**
     * Requests a native ad
     * @return void
     */
    requestNativeAd: function(story, useMainAdCodes) {
        var section = 'news';

        var options = {
                'server': 'dfp',
                'size': '2x2',
                'targets': {}
            };

        if(story) {
            var storyData;

            if(story.data.main_ad_codes && useMainAdCodes) {
                storyData = story.data.main_ad_codes;
            } else {
                storyData = story.data.ad_codes;
            }

            if(storyData) {
                section = storyData.ad;

                options.targets.pr = Pheme.Config.fetchCurrentMarket().adCode;
                options.targets.loc = storyData.loc;
                options.targets.imp = storyData.imp;
                options.targets.page = storyData.page;
                options.targets.ck = storyData.ck;
                options.targets.nk = storyData.nk;

                if(storyData.sck && storyData.sck.length > 0) {
                    options.targets.sck = storyData.sck;
                }
            }
        }
        
        options.id = '/3081/' + Pheme.Config.fetchCurrentMarket().adCode + '.' + (Ext.os.is.iOS ? 'ip' : 'aroid') + '/' + section;

        if(Pheme.helpers.DfpAds.getTestAds()) {
            options.targets.test = 'yes';
        }

        if(Pheme.helpers.DfpAds.getSampleAds()) {
            options.size = '1x1';
            options.id = '/7814368/NativeAdSamples';
        }

        Pheme.helpers.PolarAdsHelper.requestAd(options, 'Main List');
    },
    /**
     * [showSingleStoryView description]
     * @param  {[type]} storyId [description]
     * @param  {[type]} store   [description]
     * @return {[type]}         [description]
     */
    showSingleStoryView: function(storyId, store) {
        var alertStoryView = Ext.getCmp('AlertStoryView');
        
        Pheme.app.currentStoryId = storyId;
        Pheme.app.currentStoryStore = store;

        alertStoryView.setStoryId(storyId);
        alertStoryView.setStoreId(store);
        
        if(Ext.Viewport.getActiveItem().getItemId() == 'AlertStoryView'){
            alertStoryView.hide();
            alertStoryView.show();
        }

        Pheme.helpers.Common.switchView('#AlertStoryView');
    },

    /**
     * Auto refresh story/content lists to check for changes to content with WP
     */
    autoRefreshApp: function() {
        // TODO: Check for new content

        // reload read stories to check for new content (updates..etc)
        this.requestUpdatedStoryData();
    },

    /**
     * Request a list of updated content from WP for stories that have been read & are your favorite
     */
    requestUpdatedStoryData: function() {
        var me = this;
        var getRemoteStoryIdList = [];
        var readStore = Ext.getStore("ReadStory");
        readStore.sort('date', 'DESC');
        var favoriteStore = Ext.getStore("FavoriteStories");
        favoriteStore.sort('date', 'DESC');

        /////  READ STORIES /////
        /////////////// LIMIT BY TIME
        // // only request for updates for stories that have been read in the last 24hrs
        // var cutOffDate = new Date();
        // cutOffDate.setDate(cutOffDate.getDate()-1);

        // readStore.each(function(item, index, length) {
        //     if(item.data.date > cutOffDate)
        //         getRemoteStoryIdList.push(item.internalId.toString());
        // });
        /////////////// LIMIT BY TIME

        /////////////// LIMIT BY COUNT
        // set a limit of x read stories to check for updates (performance issue)
        var maxReadStoriesToCheck = 10;
        readStore.each(function(item, index, length) {
            if(maxReadStoriesToCheck == 0) return;

            if(getRemoteStoryIdList.indexOf(item.internalId.toString()) < 0) {
                getRemoteStoryIdList.push(item.internalId.toString());
                maxReadStoriesToCheck--;
            }
        });
        /////////////// LIMIT BY COUNT

        ///// FAVORITE STORIES /////
        // I know all favorites would have been read but...
        // we need to see if any of our favorites have been purged from ReadStory store
        // then we can check for updates to those as well

        // set a limit of x additional items to check for updates
        var maxAdditionalFavoritesToCheck = 10;
        favoriteStore.each(function(item, index, length) {
            if(maxAdditionalFavoritesToCheck == 0) return;

            if(getRemoteStoryIdList.indexOf(item.internalId.toString()) < 0) {
                getRemoteStoryIdList.push(item.internalId.toString());
                maxAdditionalFavoritesToCheck--;
            }
        });

        // TODO: Create/Call function that only returns a list of updated content (compare pub date)
        // modifications to api and plugin required
        Pheme.helpers.Common.getRemoteStories(getRemoteStoryIdList, function(data, addedStories) {
            // if stories have been updated then update UI
            if(data.length > 0) {
                // only request a new native ad if there are new stories in the list
                if(addedStories.length > 0) {
                    var storiesList = Ext.getCmp('storiesList'),
                        story = storiesList.getStore().first();

                    Pheme.helpers.Common.requestNativeAd(story, true);
                }

                me.refreshApp();
            }
        });
    },

    /** 
     * Refresh/reload all app content and other UI elements
     */
    refreshApp: function() {
        // refresh content lists
        this.refreshContentLists();

        // refresh favorite count icon
        Pheme.helpers.FavoriteStories.updateFavoriteUpdatedCount();

        // TODO: refresh topics
    },

    /**
     * Refresh story/content lists when changes to content may have occurred
     * @return {[type]} [description]
     */
    refreshContentLists: function() {
        var storiesList = Ext.getCmp('storiesList');
        var searchList = Ext.getCmp('searchResult');
        var favoriteView = Ext.getCmp('FavoriteView');
        var activeId = Ext.Viewport.getActiveItem().id;

        // update main view - if view is not active
        if(storiesList && activeId != 'mainViewport' && activeId != 'NativeAdView') {
            storiesList.setScrollToTopOnRefresh(false);
            storiesList.refresh();
            // if we don't set this back the pull to refresh gets screwed up
            storiesList.setScrollToTopOnRefresh(true);
        }
        
        // update search view - if view is not active
        if(searchList && activeId != 'SearchView') {
            searchList.setScrollToTopOnRefresh(false);
            searchList.refresh();
        }

        // update favorite popup view
        if(!favoriteView.isHidden()) {
            Pheme.helpers.FavoriteStories.showFavoriteView();
        }
    },

    /**
     * Central view animator.
     * @param object view View to animate
     * @param array options
     */
    switchView: function(view, options) {

        if(Ext.os.is('Android')) {
            Ext.Viewport.setActiveItem(view);
        }
        else {
            if(!options) {
                options = { type: 'slide', direction: 'left', duration: 250 };
            } 
            else if(typeof options.type == "undefined"){
                options = { type: 'slide', direction: options.direction, duration: 250 }
            }

            Ext.Viewport.animateActiveItem(view, options);
        }

    },

    /**
     * Central panel/element animator.
     * @param object element Element to animate
     * @param bool out Direction of animation [true|false]
     */
    switchElement: function(element, out, animate) {
        var animate = (animate === undefined) ? true : animate;

        //if(Ext.os.is('Android')) animate = false;

        if(out === true) {
            element.removeCls('in in-no-animation');

            if(animate) {
                element.addCls('out');
            }
            else {
                element.addCls('out-no-animation');
            }
        } 
        else {
            element.removeCls('out out-no-animation');

            if(animate) {
                element.addCls('in');
            }
            else {
                element.addCls('in-no-animation');
            }
        }
    },

    /**
     * Check if a story has been updated since user last read it
     * @param int storyId
     * @param date last_updated
     * @return bool
     */
    checkIfUpdated: function(storyId, last_updated) {
        try { 
            if(!storyId) return false;

            var readStories = Ext.getStore("ReadStory");
            var read = readStories.getById(storyId);

            if(!read) return false;

            return (read.data.date < last_updated);
        } 
        catch(e) { return false; }
    },

    /**
     * Set date when a story has been last read
     * @parm int storyId
     * @return bool
     */
    updateReadStory: function(storyId) {
        try {
            if(!storyId) return false;

            var readStore = Ext.getStore("ReadStory");
            var read = readStore.getById(storyId);
            var newDate = new Date();
            var me = this;

            if(read) {
                read.set('date', newDate);
            }
            else {
                var readStory = Ext.create('Pheme.model.ReadStory', {
                        id: storyId,
                        date: newDate
                    });

                readStory.setDirty(true);

                readStore.add( readStory );
                readStore.sync();
            }
            
            this.getStory(storyId, function(story) {
                story.set('updated', false);

                // always try and refresh views if we have read something
                // this will handle updates to 'updated' flag and any other things we may need to refresh
                me.refreshApp();
            });

            

            return true;
        }
        catch(e) { return false; }
    },

    /**
     * Get specific stories from Remote and add them to local 'Stories' store
     * @param array Array of ids
     * @param function callback( array Story )
     */
    getRemoteStories: function(storyIdList, callback) {

        if(storyIdList.length == 0) {
            if(typeof callback != 'undefined')
                callback([]);
        }

        var storyIdString = "";
        var storyStore = Ext.getStore("Stories");
        var centralStore = Ext.getStore("CentralStories");
        var readStore = Ext.getStore("ReadStory");
        var favoriteStore = Ext.getStore("FavoriteStories");

        for(var i=0;i<storyIdList.length;i++) {
            storyIdString += storyIdList[i] + ",";
        }

        if(storyIdString == "") return;

        // call 'updates' with string of comma separated id's
        Ext.data.JsonP.request({
            url: Pheme.Config.fetchCurrentMarket().endpoint + '/updates',
            method: 'GET',
            params: {
                id: storyIdString
            },
            callbackKey: '',
            success: function(result, request) {
                var storyList = [];
                var addedStoryList = [];
                var dataList = result;

                if(dataList.length == 0 ) return;
                
                for(var i=0; i<dataList.length; i++) {
                    var data = dataList[i];
                    
                    if(data !== null) {
                        // get existing story if it exists
                        // set the time (which we use for sorting) of the requested story to 00:00:01
                        // that way an individual story won't jump to the top because we currently
                        // do not know its position in the order
                        var existing = storyStore.getById(data.id),
                            new_date_sort = new Date(data.date_sort);

                        new_date_sort.setHours(0, 0, 1);

                        // if there is an existing copy locally use the sort date from it
                        // The main list will not re-order content this way, but if the user
                        // pulls to refresh or foregrounds the app it will re-order - this is ok
                        if(existing) {
                           new_date_sort = existing.data.date_sort;
                        }

                        // create new story object
                        var newStory = Ext.create('Pheme.model.Story', {
                            id: parseInt(data.id),
                            title: data.title,
                            content: data.content,
                            excerpt: data.excerpt,
                            categories: data.categories,
                            featured_image_url: data.featured_image_url,
                            featured_image_url_x2: data.featured_image_url_x2,
                            thumbnail_url: data.thumbnail_url,
                            thumbnail_url_x2: data.thumbnail_url_x2,
                            permalink: data.permalink,
                            last_modified: data.last_modified,
                            date: data.date,
                            date_utc: data.date_utc,
                            date_sort: new_date_sort,
                            post_format: data.post_format,
                            updated: data.updated,
                            no_thumbnail: data.no_thumbnail,
                            caption: data.caption,
                            credit: data.credit,
                            ad_codes : data.ad_codes,
                            main_ad_codes: data.main_ad_codes
                        });

                        newStory.setDirty(true);

                        storyList.push(newStory);
                    } else {
                        //clear stores
                        var readObj = readStore.getById(storyIdList[i]);
                        var storyObj = storyStore.getById(storyIdList[i]);
                        var centralObj = centralStore.getById(storyIdList[i]);
                        var favObj = favoriteStore.getById(storyIdList[i]);
                        
                        if (readObj != null){
                            readStore.remove(readObj);
                            readStore.sync();
                        }
                        
                        if (storyObj != null){
                            storyStore.remove(storyObj);
                            storyStore.sync();
                        }

                        if (centralObj != null){
                            centralStore.remove(centralObj);
                            centralStore.sync();
                        }
                        if (favObj != null) {
                            favoriteStore.remove(favObj);
                            favoriteStore.sync();
                        }

            
                    }
                }

                // update our stores with newly updated data
                for(var j=0; j<storyList.length; j++) {
                    var s = storyList[j];

                    // add will perform an update if it exists
                    try {
                        // only update stories that exist in the story store
                        // other stories will go to central store
                        var existingIndex = storyStore.find('id', s.internalId);

                        if(existingIndex >= 0) {
                            s.data.index = existingIndex;
                            storyStore.add(s);
                        }
                        else {
                            var existingCentralIndex = centralStore.find('id', s.internalId);
                            
                            // if story does not exist in central - add to our list
                            if(existingCentralIndex < 0) {
                                addedStoryList.push(s);
                            }

                            centralStore.add(s);
                        }
                    }
                    catch(e) {
                        console.log(e);
                    }
                }

                if(typeof callback != 'undefined')
                    callback(storyList, addedStoryList);
            }
        });
    },

    /**
     * Iterate over each record in a store and assign it an index
     * Used for sorting our lists
     * @param {[type]} store [description]
     */
    setStoryStoreRecordIndexes: function(store) {
        if(!store.getIsIndexing()) {
            store.setIsIndexing(true);

            store.each(function(record, index) {
                record.set('index', index);
            },
            store
            );

            store.setIsIndexing(false);
        }
    },

    /**
     * Get a specific story
     * @param  {Function} callback [description]
     * @return {[type]}            [description]
     */
    getStory: function(id, callback) {
        var storyStore = Ext.getStore("Stories");
        var centralStore = Ext.getStore("CentralStories");

        var story = storyStore.getById(id);

        // look in central store for the story
        if(!story) {
            story = centralStore.getById(id);
        }

        if(story) {
            if(typeof callback != 'undefined') {
                callback(story);
            }
        }
        else {
            Pheme.helpers.Common.getRemoteStories([id], function(data, addedStories) {
                if(data.length > 0 && typeof data[0] != 'undefined') {
                    callback(data[0]);
                }
            });
        }
    },

    createLoadingIndicator: function() {
        Ext.Viewport.add({
            xtype: 'loadmask',
            id: 'loadingMask',
            message: 'Loading...',
            indicator: true,
            hidden: true,
            margin: '70 0 0 0'
        });
    }
});