/**
 *  
 *  Urban Airship Alerts Helper
 *  @class Pheme.helpers.UaAlerts
 *  Description: This class encapsulates the functionality from the Urban Airship PhoneGap Plugin.
 *  It was made a singleton as it makes sense to only have one instance that manages notifications.
 */
Ext.define('Pheme.helpers.UaAlerts', {
	singleton: true,
	config: {
        pushInst: (function(){
            var pushPlugin;

            //Use this to prevent error from crash app when developing in browser.
            try {
                pushPlugin = PushNotification;
            } catch(e) {
                pushPlugin = undefined;
            }
            return pushPlugin;
        })(),

		//Debug mode for testing purposes
		debugMode: false,

		//Show debugging info in alert dialogs (for testing on device where console.logs aren't visible)
		showDebugInAlerts: false,

    },
           
   //Check to see if Push was enabled by user
   isPushEnabled: function() {
        var pushInst = this.getPushInst(),
            debugMode = this.getDebugMode(),
            showAlerts = this.getShowDebugInAlerts();

        if(pushInst) {
            var isEnabled = PushNotification.isPushEnabled(function(enabled) {
                if(enabled) {
                    if(debugMode) {
                        if(showAlerts) {
                            alert('Push notifications enabled');
                        } else {
                            console.log('Push notifications enabled');
                        }
                    }
                    
                    return true;
                } else {
                    if(debugMode) {
                        if(showAlerts) {
                            alert('Push notifications NOT enabled!');
                        } else {
                            console.log('Push notifications NOT enabled!');
                        }
                    }
                    return false;
                }
            });
           
           return isEnabled;
        } else {
           return false;
        }
    },
    /**
     * [registerForPushNotifications description]
     * @param  {Function} callback [description]
     * @return {[type]}            [description]
     */
	registerForPushNotifications: function(callback) {
           
        var pushInst = this.getPushInst(),
            debugMode = this.getDebugMode(),
            showAlerts = this.getShowDebugInAlerts();

		if(pushInst) {
			//Add listener to document object
			document.addEventListener('urbanairship.push', function(event) {
    			callback(event, debugMode, showAlerts);
			});

			return true;
		} else {
			return false;
		}
	},
    /**
     * [registerDeviceRegistration description]
     * @param  {Function} callback [description]
     * @return {[type]}            [description]
     */
	registerDeviceRegistration: function(callback) {

       var pushInst = this.getPushInst(),
           debugMode = this.getDebugMode(),
           showAlerts = this.getShowDebugInAlerts();
           
		if(pushInst) {

			//Add listener to document object
			document.addEventListener('urbanairship.registration', function(event) {
				callback(event, debugMode, showAlerts);
			});

			return true;
		} else {
			return false;
		}
	},
    /**
     * [setTags description]
     * @param {[type]} tagValues [description]
     */
    setTags: function(tagValues) {
        if(!tagValues) {
           return false;
        }

        return PushNotification.getTags(function(obj){
            if(obj.tags) {

                if(Array.isArray(tagValues)) {
                    PushNotification.setTags(tagValues);
                }

                if(typeof tagValues === 'string') {
                    var tags = obj.tags;

                    if(tags.indexOf(tagValues) < 0) {
                        tags.push(tagValues);
                        PushNotification.setTags(tags);
                    }
                }
            } else {
                return false;
            }
        });
    },
    /**
     * [removeTag description]
     * @param  {[type]} tag [description]
     * @return {[type]}     [description]
     */
    removeTag: function(tag) {
        if(!tag) {
           return false;
        }
        
        var pushInst = this.getPushInst();
        
        if(pushInst) {
            PushNotification.getTags(function(obj) {
                var removeIndex = obj.tags.indexOf(tag);
                 
                if(removeIndex > -1) {
                    obj.tags.splice(obj.tags.indexOf(tag), 1);
                    PushNotification.setTags(obj.tags);
                }
            });
        } else {
            return false;
        }
    },
    /**
     * [getIncomingPush description]
     * @param  {Function} callback [description]
     * @param  {[type]}   eventId  [description]
     * @return {[type]}            [description]
     */
    getIncomingPush: function(callback, eventId) {
        if(!callback) {
            return null;
        }
           
        var pushInst = this.getPushInst(),
            debugMode = this.getDebugMode(),
            showAlerts = this.getShowDebugInAlerts();
    
        if(pushInst) {
            pushInst.getIncoming(function(incoming) {
                callback(incoming, debugMode, showAlerts, eventId);
            });
        } else {
            return false;
        }
    },
    /**
     * [constructor description]
     * @param  {[type]} config [description]
     * @return {[type]}        [description]
     */
    constructor: function(config) {
        this.initConfig(config);
    },
    /**
     * [init description]
     * @return {[type]} [description]
     */
    init: function() {
        this.callParent(arguments);
    }
});