/**
 *  Main
 *  @class Pheme.view.Main
 *  @extends Ext.Container
 *  Description: ...
 */
Ext.define('Pheme.view.Main', {
    extend: 'Ext.Container',
    requires: ['Pheme.plugin.CustomListPaging', 'Ext.plugin.PullRefresh', 'Ext.dataview.List', 'Ext.Img', 'Pheme.view.CustomList'],
    xtype: 'Main',
    id: 'MainView',
    config: {
        cls: 'slide',
        width: '100%',
        layout: 'vbox',
        items: [
            {
                xtype: 'titlebar',
                items: [
                    {
                        xtype: 'image',
                        cls: 'icon-menu',
                        id: 'nav_btn',
                        src: 'resources/images/toolbar/top-menu.png',
                    },
                    {
                        xtype: 'image',
                        cls: 'icon-logo',
                        align: 'center',
                        id: 'main_nav_icon',
                        src: 'resources/images/toolbar/'+ Pheme.Config.fetchCurrentMarket().resourcesDirectory +'/logo-main.png',
                    },
                    {
                        xtype: 'label',
                        id: 'followingCountTitleBar',
                        cls: 'follow-count-icon',
                        hidden: true,
                        listeners: {
                            painted: function() {
                                Pheme.helpers.FavoriteStories.updateFavoriteUpdatedCount(); 
                            }
                        }
                    },
                    {
                        xtype: 'image',
                        cls: 'alerts-active',
                        src: 'resources/images/toolbar/alerts-breaking.png',
                        itemId: 'btn_alert',
                    }
                ]
            }, 
    		{
                xtype: 'panel',
                style: 'background-color:white',
                cls: 'filter-bar',
                layout: 'hbox',
                pack: 'justify',
                hidden: true,
                filterSlug: undefined,
                id: 'filter_bar',
                items: [{
                    xtype: 'label',
                    cls: 'topics-label',
                    flex: 3
                }, {
                    xtype: 'image',
                    flex: 1,
                    html: 'Clear filter',
                    cls: 'clear-filter',
                    itemId: 'clear-filter',
                    listeners: {
                        tap: function(clearbtn) {
                            var store = Ext.getStore('Stories');
                            var currentURL = store.getProxy().getUrl();

                            Pheme.app.filterActive = false;
                            
                            store.getProxy().setUrl(Pheme.Config.fetchCurrentMarket().endpoint + '/stories');
                            store.loadPage(1, {
                                callback: function(records, operation, success) {
                                    if (success) {
                                        clearbtn.parent.hide();
                                        var list = Ext.getCmp('storiesList')
                                        list.removeCls("filtered");
                                        list.getScrollable().getScroller().scrollTo(0, 0);

                                        Pheme.helpers.Common.requestNativeAd(records[0], true);
                                    }
                                    
                                    Pheme.helpers.Omniture.pageEvent("Clear Filter Click", "main view", { section: currentURL.substring(currentURL.lastIndexOf("/")+1, currentURL.length) });
                                }
                            }, this);
                        }
                    }
                }]
            }, {
                xtype: 'customlist',
                id: 'storiesList',
                flex: 1,
                useSimpleItems: true,
                disableSelection: true,
                scrollToTopOnRefresh: true,
                grouped: true,
    			html: '<div id="offline">An Internet connection is needed to access the ' + Pheme.Config.fetchCurrentMarket().aboutUs + ' app. Please try again.</div>',
                bufferSize: 3,
                store: 'Stories',
                scrollable: {
                    momentumEasing: {
                        momentum: {
                            acceleration: 10,
                            friction: 0.7
                        },
                        bounce: {
                            acceleration: 10,
                            springTension: 0.3
                        }
                    }
                },
                listeners: {
                    refresh: function() {
                    },
                    painted: function() {
                        // delay page track so if this get called a few times it should only call once because of the delay
                        if(Pheme.app.mainViewTrackingTimeout == null && Ext.Viewport.getActiveItem().id == 'mainViewport') {
                            Pheme.app.mainViewTrackingTimeout = setTimeout(function() {
                                Pheme.helpers.Omniture.pageView("main view");
                                Pheme.app.mainViewTrackingTimeout = null;
                            }, 1000);
                        }
                    }
                },
                plugins: [
                    {
                        id: 'pullRefresh',
                        xclass: 'Ext.plugin.PullRefresh',
                        pullText: 'Pull down to refresh...',
                        scrollerAutoRefresh: true,
                        autoSnapBack: false,
                        listeners: {
                            'prefetchlatest' : function() {
                                // clear stories store and reload from the top
                                var store = Ext.getStore('Stories');
                                store.data.clear();
                                store.sync();
                            },
                            'latestfetched' : function(pullRefresh) {

                                pullRefresh.lastUpdated = new Date();
                               
                                // this is to keep store.currentPage at 1
                                var store = Ext.getStore('Stories');
                                store.loadPage(1);

                                // check if we are online
                                var isOnline = true;
                                if (Ext.os.is('Android')) {
                                    isOnline = Ext.device.Connection.isOnline();
                                }
                                else {
                                    isOnline = window.navigator.onLine;
                                }

                                // update online / offline message
                                var offlineEl = document.getElementById("offline");
                                if(!isOnline) {
                                    if(offlineEl.style.display != "block"){
                                        Ext.getStore('Stories').removeAll();
                                        offlineEl.style.display =  "block";
                                    }
                                }
                                else if(offlineEl.style.display == "block") {
                                    offlineEl.style.display =  "none";
                                    Ext.getCmp("storiesList").getScrollable().getScroller().scrollTo(0, 0);
                                    Ext.getStore("topics").loadPage(1);
                                }

                                this.fireEvent('refreshTopics');
                            }
                        }
                    }, 
                    {
                        xclass: 'Pheme.plugin.CustomListPaging',
                        id: 'listpaging',
                        autoPaging: true,
                        bufferSize: 50
                    }
                ], 
                //Please do not auto format this document. Autoformat ruins the format on template below and makes it hard to read.
                itemTpl: [
                    '   <div class="news_item {index}">                                                                                                                   ',
                    '       <tpl if="index == 0">                                                                                                        ', 
                    '           <div class="feature-image">                                                                                                         ',
                    '               <tpl if="this.getIsRetina() && featured_image_url_x2 && !no_thumbnail && !this.isFilterActive()">                                   ',
                    '                   <img class="show-feature" src="{featured_image_url_x2}" width="640" height="384" />                                         ',
                    '               <tpl elseif="featured_image_url && !no_thumbnail  && !this.isFilterActive()">                                                       ',    
                    '                   <img class="show-feature" src="{featured_image_url}"  width="320" height="192" />                                           ',
                    '               <tpl else>                                                                                                                     ',
                    '                   <div class="no-feature-image"></div>                                                                                        ',
                    '               </tpl>                                                                                                                          ',
                    '           </div>                                                                                                                              ',
                    '       </tpl>                                                                                                                                  ',
                    '       <div class="main_headlines {[this.isFilterActive() ? "filtered-list-view" : "main-list-view"]}">             ',
                    '           <tpl if="this.getIsRetina() && thumbnail_url_x2  && !no_thumbnail">                                                                 ',
                    '                <img class="<tpl if="this.isEven(index)">even<tpl else>odd</tpl>" src="{thumbnail_url_x2}" width="190" height="190" />                                 ',
                    '           <tpl elseif="thumbnail_url && !no_thumbnail">                                                                                       ',
                    '                <img class="<tpl if="this.isEven(index)">even<tpl else>odd</tpl>" src="{thumbnail_url}" width="95" height="95" />                                      ',
                    '           </tpl>                                                                                                                              ',
                    '           <h3 class="list-headline-3"><tpl if="this.isFollowed(id)"><div class="following-star"></div></tpl>{title}</h3>                                                                                            ',    
                    '           <div class="timestamp">                                                                                                            ',
                    '                <tpl if="updated">                                                                                                             ',
                    '                    <span class="updated">Updated </span>                                                                                      ',
                    '                </tpl>                                                                                                                         ',
                    '                <tpl if="sponsor_name">                                                                                                 ',
                    '                   <span class="sponsor-line">{sponsor_prefix}: <b>{sponsor_name}</b></span>',
                    '                </tpl>                                         ',
                    '           </div>                                                                                                                             ',
                    '       </div>                                                                                                                                  ',
                    '   </div><div style="clear:both"></div>',

                    {
                        isFilterActive: function() {
                            return Pheme.app.filterActive;
                        },
                        isEven: function(index) {
                            return (index > 0 && index % 2 > 0);
                        },
                        getIsRetina: function() {
                            return Pheme.app.isRetina();
                        },
                        isFollowed: function(id) {
                            return Pheme.helpers.FavoriteStories.isFavoriteStory(id);
                        },
                        isNativeAd: function(id) {
                            var store = Ext.getStore('Stories'),
                                story = store.getById(id);

                            if(story.data.sponsor_name) {
                                return true;
                            } else {
                                return false;
                            }
                        }
                    }
                ]
            }, 
            {
                items: [{
                    xtype: 'panel',
    				id: 'MainBottomAd',
                    cls: 'bottom_ad',
                    html: '<div id="main-bottom-ad"></div>',
                    width: '100%',
                    height: '50px'
                }]

            }
        ],
        listeners: {
            initialize: function() {
                this.fireEvent('displayNativeAd');
            },
            painted: function() {
                this.fireEvent('showInterstitial');
            }
        }
    }

});
