describe('Alert Story Controller', function() {

    var app,
        fixture;

    beforeEach(function() {
        app = Ext.create('Ext.app.Application', {
            name: 'Pheme'
        });

        fixture = [{
            data: {
                ad_codes: {
                    ad: "news/local-news/story",
                    ck: "news",
                    loc: "top",
                    nk: "print",
                    page: "story",
                    pr: "oc",
                    sck: [
                        "local-news"
                    ]
                },
                caption: "",
                categories: [],
                length: 1,
                credit: "Greg Thomsen",
                date: "Mon Jul 14 2014 20:57:00 GMT-0700 (PDT)",
                date_sort: "Mon Jul 14 2014 23:59:00 GMT-0700 (PDT)",
                excerpt: "That was a month ago, and since then he has plastered signs around town offering a $150 reward for his four-armed flying friend, but to no avail.",
                featured_image_url: "http://wpmedia.ottawacitizen.com/2014/07/greg-thomsen-lost-his-quad-copter-a-month-ago-sad-days-for-0715-quad-copter.jpg?w=320&h=192&crop=1",
                featured_image_url_x2: "http://wpmedia.ottawacitizen.com/2014/07/greg-thomsen-lost-his-quad-copter-a-month-ago-sad-days-for-0715-quad-copter.jpg?w=640&h=384&crop=1",
                id: 274973,
                last_modified: "Mon Jul 14 2014 20:57:00 GMT-0700 (PDT)",
                main_ad_codes: {
                    ad: "news/index",
                    ck: "news",
                    imp: "news",
                    loc: "top",
                    nk: "print",
                    page: "index"
                },
                no_thumbnail: false,
                permalink: "http://ottawacitizen.com/storyline/hobbyist-must-watch-as-his-800-quad-copter-fly-flies-away",
                post_format: null,
                thumbnail_url: "http://wpmedia.ottawacitizen.com/2014/07/greg-thomsen-lost-his-quad-copter-a-month-ago-sad-days-for-0715-quad-copter.jpg?w=95&h=95&crop=1",
                thumbnail_url_x2: "http://wpmedia.ottawacitizen.com/2014/07/greg-thomsen-lost-his-quad-copter-a-month-ago-sad-days-for-0715-quad-copter.jpg?w=190&h=190&crop=1",
                title: "Hobbyist must watch as his $800 quad-copter fly, flies&nbsp;away",
                updated: false
            },
        }];
    });

    describe('Instantiating new Alert Story controller', function() {
        it('should create a new controller instance', function() {
            alertStoryController = Ext.create('Pheme.controller.AlertStory', { application: app });
            expect(alertStoryController).not.toBe(undefined);
        });
    });

    describe('Scrolling through Story Elements in Alert Story', function() {
        it('should call the sendAnalytics method when story element is changed', function() {

            alertStoryController = Ext.create('Pheme.controller.AlertStory', { application: app });
            Omniture = Pheme.helpers.Omniture;

            //Spy on the Omniture object
            spyOn(Omniture, 'pageView');

            Pheme.helpers.ControllerEvents.sendAnalytics(fixture);

            //Expect that the Omniture send method was fired
            expect(Omniture.pageView).toHaveBeenCalled();
        });
    });

    describe('Scrolling through Story Elements in Alert Story Controller should trigger new ad call', function() {
        it('should call the renderBannerAd method when a story element is changed', function() {
            alertStoryController = Ext.create('Pheme.controller.AlertStory', { application: app });

            spyOn(Pheme.helpers.ControllerEvents, 'renderBannerAd');

            Pheme.helpers.ControllerEvents.renderAd(fixture)

            expect(Pheme.helpers.ControllerEvents.renderBannerAd).toHaveBeenCalled();
        });
    });

    afterEach(function() {
        app = undefined;
    });

});
