describe('Generic Story Controller', function() {

    var app,
        fixture;

    beforeEach(function() {
        app = Ext.create('Ext.app.Application', {
            name: 'Pheme',
            isRetina: function() {
                return (window.devicePixelRatio > 1) ? true : false;
            }
        });

        fixture = {
            data: {
                ad_codes: {
                    ad: "news/local-news/story",
                    ck: "news",
                    loc: "top",
                    nk: "print",
                    page: "story",
                    pr: "oc",
                    sck: [
                        "local-news"
                    ]
                },
                caption: "",
                categories: [],
                length: 1,
                credit: "Greg Thomsen",
                date: "Mon Jul 14 2014 20:57:00 GMT-0700 (PDT)",
                date_sort: "Mon Jul 14 2014 23:59:00 GMT-0700 (PDT)",
                excerpt: "That was a month ago, and since then he has plastered signs around town offering a $150 reward for his four-armed flying friend, but to no avail.",
                featured_image_url: "http://wpmedia.ottawacitizen.com/2014/07/greg-thomsen-lost-his-quad-copter-a-month-ago-sad-days-for-0715-quad-copter.jpg?w=320&h=192&crop=1",
                featured_image_url_x2: "http://wpmedia.ottawacitizen.com/2014/07/greg-thomsen-lost-his-quad-copter-a-month-ago-sad-days-for-0715-quad-copter.jpg?w=640&h=384&crop=1",
                id: 274973,
                last_modified: "Mon Jul 14 2014 20:57:00 GMT-0700 (PDT)",
                main_ad_codes: {
                    ad: "news/index",
                    ck: "news",
                    imp: "news",
                    loc: "top",
                    nk: "print",
                    page: "index"
                },
                no_thumbnail: false,
                permalink: "http://ottawacitizen.com/storyline/hobbyist-must-watch-as-his-800-quad-copter-fly-flies-away",
                post_format: null,
                thumbnail_url: "http://wpmedia.ottawacitizen.com/2014/07/greg-thomsen-lost-his-quad-copter-a-month-ago-sad-days-for-0715-quad-copter.jpg?w=95&h=95&crop=1",
                thumbnail_url_x2: "http://wpmedia.ottawacitizen.com/2014/07/greg-thomsen-lost-his-quad-copter-a-month-ago-sad-days-for-0715-quad-copter.jpg?w=190&h=190&crop=1",
                title: "Hobbyist must watch as his $800 quad-copter fly, flies&nbsp;away",
                updated: false,
                content: [
                    '<p>With a general election expected next year, federal political parties are actively campaigning to raise money from supporters — but dozens of their own MPs haven’t heeded the call to give until it hurts.</p><img alt="" border="0" src="http://pixel.wp.com/b.gif?host=ottawacitizen.com&#038;blog=66829272&#038;post=299647&#038;subd=postmediaottawacitizen2&#038;ref=&#038;feed=1" width="1" height="1" />',
                    '<p>A <em>Citizen</em> analysis of Elections Canada records shows about one-third of all members of Parliament gave less than the legal maximum $1,200 to their own party, despite their six-figure salaries, and some donated <em>nothing at all</em>.</p>↵<p>The annual financial reports filed with the agency last month list every contributor who gave a total of $200 or more in the calendar year. The names of 22 current MPs are missing from the lists, suggesting they gave less than $200 or made no contributions.</p><img alt="" border="0" src="http://pixel.wp.com/b.gif?host=ottawacitizen.com&#038;blog=66829272&#038;post=299647&#038;subd=postmediaottawacitizen2&#038;ref=&#038;feed=1" width="1" height="1" />'
                ]
            }
        };

        customCarousel = Ext.create('Pheme.view.CustomCarousel', { application: app });
        genericStoryController = Ext.create('Pheme.controller.GenericStory', { application: app });
    });

    describe('adding a story to the main carousel', function() {
        it('should return a carousel with items', function() {
            var carousel = genericStoryController.addStory(fixture);

            expect(carousel.getItems().length).toBeGreaterThan(0);
        });

        it('should call the customCarouselFactory method', function() {
            spyOn(genericStoryController, 'customCarouselFactory').and.callThrough();

            genericStoryController.addStory(fixture);

            expect(genericStoryController.customCarouselFactory).toHaveBeenCalled();
        });

        it('should call the carouselPanelFactory method', function() {
            spyOn(genericStoryController, 'carouselPanelFactory').and.callThrough();

            genericStoryController.addStory(fixture);

            expect(genericStoryController.carouselPanelFactory).toHaveBeenCalled();
        });
    });

    describe('creating a custom carousel', function() {
        it('should return a custom carousel', function() {
            var carousel = genericStoryController.customCarouselFactory(274973);

            expect(carousel).not.toBe(undefined);
            expect(carousel.internalId).toEqual(274973);
        });

        it('should call Ext.Create to create the Carousel', function() {
            spyOn(Ext, 'create').and.callThrough();
            genericStoryController.customCarouselFactory(274973);
            expect(Ext.create).toHaveBeenCalled();
        });

        it('should still return a custom carousel even if there is no internal id passed in', function() {
            var carousel = genericStoryController.customCarouselFactory();
            expect(carousel).not.toBe(undefined);
        });

        it('should return a panel for the custom carousel', function() {
            var carouselPanel = genericStoryController.carouselPanelFactory();

            expect(carouselPanel).not.toBe(undefined);
        });

        it('should create a panel for the content of the story', function() {
            var carouselPanel = genericStoryController.carouselPanelFactory(),
                contentPanel = genericStoryController.generateContentPanel(fixture, carouselPanel);

            expect(contentPanel).not.toBe(undefined);
            expect(contentPanel.getItems().length).toBeGreaterThan(0);
        });

        it('should add contents to carousel panel with pagination', function() {
            var carouselPanel = genericStoryController.carouselPanelFactory(),
                contentPanel = genericStoryController.generateContentPanel(fixture, carouselPanel);

            carouselPanel = genericStoryController.addContentPanelsToCarousel(fixture, carouselPanel, contentPanel);

            expect(carouselPanel.getItems().length).toBeGreaterThan(0);
            expect(carouselPanel.getItems().length).toEqual(2);
        });

        it('should create a featured image panel', function() {
            var featuredImagePanel = genericStoryController.imagePanelFactory(fixture);

            expect(featuredImagePanel).not.toBe(undefined);
        });

        it('should create an image caption panel', function() {
            var imageCaptionPanel = genericStoryController.imageCaptionPanelFactory(fixture);

            expect(imageCaptionPanel).not.toBe(undefined);
        });

        describe('checking for featured image', function() {
            beforeEach(function() {
                noFeaturedFixture = {
                    data: {
                        featured_image_url: ""
                    }
                }
            });

            it('should return true if there is a featured image in the story object', function() {
                var featuredStory = genericStoryController.checkIfFeaturedStoryExists(fixture);

                expect(featuredStory).toBe(true);
            });

            it('should return false if there is no featured image in the story object', function() {
                var featuredStory = genericStoryController.checkIfFeaturedStoryExists(noFeaturedFixture);

                expect(featuredStory).toBe(false);
            });

            afterEach(function() {
                noFeaturedFixture = null;
            });
        });

        describe('checking for gallery format of post', function() {
            beforeEach(function() {
                galleryStory = {
                    data: {
                        post_format: "gallery"
                    }
                }
            });
            it('should return true if format of post is gallery', function() {
                var isGallery = genericStoryController.checkIfGallery(galleryStory);

                expect(isGallery).toBe(true);
            });

            afterEach(function() {
                noFeaturedFixture = null;
            });
        });

        describe('checking for credit property', function() {
            beforeEach(function() {
                noCreditStory = {
                    data: {}
                }
            });

            it('should return true if there is a credit to associate an image or images with', function() {
                var isCredit = genericStoryController.checkIfCreditExists(fixture);

                expect(isCredit).toBe(true);
            });

            it('should return false if there is no credit', function() {
                var isCredit = genericStoryController.checkIfCreditExists(noCreditStory);

                expect(isCredit).toBe(false);
            });

            afterEach(function() {
                noCreditStory = null;
            });
        });

        describe('Creating story content panels for the carousel', function() {
            it('should return a panel with 2 children panels', function() {
                var pageIndex = 0,
                    storyContentPanel = genericStoryController.generateStoryContentPanel(fixture.data.content, pageIndex);

                expect(storyContentPanel.getItems().length).toEqual(2);
            });

            it('should add content panels to the main carousel', function() {
                var carousel = genericStoryController.customCarouselFactory(fixture.data.id),
                    previousItemCount = carousel.getItems().length;

                genericStoryController.addStoryContentPanelsToCarousel(carousel, fixture.data.content);

                expect(carousel.getItems().length).toBeGreaterThan(previousItemCount);
            });
        });

        describe('setting properties on the carousel', function() {
            it('should set the current story id to the id of the story passed in', function() {
                var carousel = genericStoryController.customCarouselFactory(fixture.data.id);

                genericStoryController.setCarouselProperties(carousel, fixture);

                expect(carousel.currentStoryId).toEqual(fixture.data.id);
            });

            it('should set the story title of the carousel to the title of the story passed in', function() {
                var carousel = genericStoryController.customCarouselFactory(fixture.data.id);

                genericStoryController.setCarouselProperties(carousel, fixture);

                expect(carousel.currentStoryTitle).toEqual(fixture.data.title);
            });

            it('should set the story date of the carousel to the date of the story passed in', function() {
                var carousel = genericStoryController.customCarouselFactory(fixture.data.id);

                genericStoryController.setCarouselProperties(carousel, fixture);

                expect(carousel.currentStoryDate).toEqual(Ext.util.Format.date(fixture.data.date, "M j, Y g:i A"));
            });

            it('should set the share message of the carousel to the title of the story passed in', function() {
                var carousel = genericStoryController.customCarouselFactory(fixture.data.id);

                genericStoryController.setCarouselProperties(carousel, fixture);

                expect(carousel.shareMessage).toEqual(fixture.data.title);
            });

            it('should set the share image of the carousel to the featured image of the story passed in', function() {
                var carousel = genericStoryController.customCarouselFactory(fixture.data.id);

                genericStoryController.setCarouselProperties(carousel, fixture);

                expect(carousel.shareImage).toEqual(fixture.data.featured_image_url);
            });

            it('should set the share link of the carousel to the permalink of the story passed in', function() {
                var carousel = genericStoryController.customCarouselFactory(fixture.data.id);

                genericStoryController.setCarouselProperties(carousel, fixture);

                expect(carousel.shareLink).toEqual(fixture.data.permalink);
            });
        });

    });

    describe('Checking if current OS is iOS', function() {
        it('should return true if current OS is iOS', function() {
            Ext.os.is['iOS'] = true;
            var isIOS = genericStoryController.checkIfOSisIOS();

            expect(isIOS).toBe(true);
            Ext.os.is['iOS'] = false;
        });

        it('should return false if the current OS is not iOS', function() {
            var isIOS = genericStoryController.checkIfOSisIOS();

            expect(isIOS).toBe(false);
        });
    });

    /*describe('Checking content for iFrames', function() {
        it('should return true if there is one or more iFrame elements present', function() {
            var contentPanel = {
                dom: {}
            };

            contentPanel.dom.innerHTML = '<body><p><iframe src="youtu.be/?191131"></iframe></p></body>';
            var iFramesExist = genericStoryController.checkForIFrames(contentPanel);

            expect(iFramesExist).toBe(true);
        });
    });*/

    describe('Following a story from the button on the story panel', function() {
        it('should call helper method to trigger following of story', function() {
            spyOn(Pheme.helpers.FollowStories, 'toggleFollow');

            genericStoryController.toggleFollowStory(fixture.data.id);

            expect(Pheme.helpers.FollowStories.toggleFollow).toHaveBeenCalled();
        });

        /*it('should get the follow button on the panel', function() {
            var followButton = genericStoryController.getFollowButtonElement(parent);
            expect(followButton).not.toBe(undefined);
        });*/

        it('should add the following css class if the story is being followed', function() {
            var mockFollowButtonElement = {
                    cls: '',
                    addCls: function(cssClass) {
                        this.cls = cssClass;
                    }
                },
                isFollowingStory = true;

            genericStoryController.toggleFollowingCSSClass(mockFollowButtonElement, isFollowingStory);
            expect(mockFollowButtonElement.cls).toEqual('inner-following');
        });

        it('should remove the following css class if the story is not being followed', function() {
            var mockFollowButtonElement = {
                    cls: 'inner-following',
                    removeCls: function(cssClass) {
                        this.cls = '';
                    }
                },
                isFollowingStory = false;

            genericStoryController.toggleFollowingCSSClass(mockFollowButtonElement, isFollowingStory);
            expect(mockFollowButtonElement.cls).toEqual('');
        });
    });

    afterEach(function() {
        app = undefined;
        genericStoryController = undefined;
        customCarousel = undefined;
    });
});
