describe('Story Controller', function() {

    var app,
        fixture;

    beforeEach(function() {
        app = Ext.create('Ext.app.Application', {
            name: 'Pheme',
            adIndex: 0,
            startIndex: 0,
            isRetina: function() {
                return true;
            }
        });

        fixture = [{
            data: {
                ad_codes: {
                    ad: "news/local-news/story",
                    ck: "news",
                    loc: "top",
                    nk: "print",
                    page: "story",
                    pr: "oc",
                    sck: [
                        "local-news"
                    ]
                },
                caption: "",
                categories: [],
                length: 1,
                credit: "Greg Thomsen",
                date: "Mon Jul 14 2014 20:57:00 GMT-0700 (PDT)",
                date_sort: "Mon Jul 14 2014 23:59:00 GMT-0700 (PDT)",
                excerpt: "That was a month ago, and since then he has plastered signs around town offering a $150 reward for his four-armed flying friend, but to no avail.",
                featured_image_url: "http://wpmedia.ottawacitizen.com/2014/07/greg-thomsen-lost-his-quad-copter-a-month-ago-sad-days-for-0715-quad-copter.jpg?w=320&h=192&crop=1",
                featured_image_url_x2: "http://wpmedia.ottawacitizen.com/2014/07/greg-thomsen-lost-his-quad-copter-a-month-ago-sad-days-for-0715-quad-copter.jpg?w=640&h=384&crop=1",
                id: 274973,
                last_modified: "Mon Jul 14 2014 20:57:00 GMT-0700 (PDT)",
                main_ad_codes: {
                    ad: "news/index",
                    ck: "news",
                    imp: "news",
                    loc: "top",
                    nk: "print",
                    page: "index"
                },
                no_thumbnail: false,
                permalink: "http://ottawacitizen.com/storyline/hobbyist-must-watch-as-his-800-quad-copter-fly-flies-away",
                post_format: null,
                thumbnail_url: "http://wpmedia.ottawacitizen.com/2014/07/greg-thomsen-lost-his-quad-copter-a-month-ago-sad-days-for-0715-quad-copter.jpg?w=95&h=95&crop=1",
                thumbnail_url_x2: "http://wpmedia.ottawacitizen.com/2014/07/greg-thomsen-lost-his-quad-copter-a-month-ago-sad-days-for-0715-quad-copter.jpg?w=190&h=190&crop=1",
                title: "Hobbyist must watch as his $800 quad-copter fly, flies&nbsp;away",
                updated: false
            }
        }];

        genericStoryConroller = Ext.create('Pheme.controller.GenericStory', { application: app });
        storyController = Ext.create('Pheme.controller.Story', { application: app });
    });

    describe('Instantiating new Story controller', function() {
        it('should create a new controller instance', function() {
            expect(storyController).not.toBe(undefined);
        });
    });

    describe('Scrolling through Story Elements in Story Controller', function() {
        it('should call the sendAnalytics method when story element is changed', function() {

            Omniture = Pheme.helpers.Omniture;

            //Spy on the Omniture object
            spyOn(Omniture, 'pageView');

            Pheme.helpers.ControllerEvents.sendAnalytics(fixture);

            //Expect that the Omniture send method was fired
            expect(Omniture.pageView).toHaveBeenCalled();
        });

        it('should track navigation when the user swipes to a story', function() {
            Omniture = Pheme.helpers.Omniture;

            //Spy on the Omniture object
            spyOn(Omniture, 'pageView');

            storyController.trackUserInteractions(fixture);

            expect(Omniture.pageView).toHaveBeenCalled();
        });

        it('should call storyAction when it is not the first view of the story', function() {
            Omniture = Pheme.helpers.Omniture;
            var firstView = false;

            //Spy on the Omniture object
            spyOn(Omniture, 'storyAction');

            storyController.trackUserNavigationIfFirstView(firstView, fixture);

            expect(Omniture.storyAction).toHaveBeenCalled();

        });
    });

    describe('Ad Operations while scrolling through Story Elements in Story Controller', function() {
        beforeEach(function() {

            dfpInstanceMock = function() {};

            dfpInstanceMock.prototype.createBannerAd = function(options, adSuccessCallback, error) {
                return true;
            };

            dfpInstanceMock.prototype.debugMode = function(options) {
                return true;
            };

            dfpInstanceMock.prototype.removeAd = function(debugObj, callback) {
                return true;
            };

            dfpInst = new dfpInstanceMock();

            dfpAdHelper = Pheme.helpers.DfpAds;

            dfpAdHelper.setDfpPluginInstance(dfpInst);
        });

        it('should render a new ad when a story element is changed', function() {

            spyOn(dfpAdHelper, 'renderBannerAd').and.callThrough();
            spyOn(dfpAdHelper, 'createDfpAd').and.callThrough();
            spyOn(dfpInst, 'createBannerAd');

            Pheme.helpers.ControllerEvents.renderBannerAd(fixture[0]);

            expect(dfpAdHelper.renderBannerAd).toHaveBeenCalled();
        });

        it('should remove existing ads when a story element is changed', function() {
            spyOn(dfpAdHelper, 'removeAds').and.callThrough();
            spyOn(dfpInst, 'removeAd');

            Pheme.helpers.ControllerEvents.renderBannerAd(fixture[0]);

            expect(dfpAdHelper.removeAds).toHaveBeenCalled();
            expect(dfpInst.removeAd).toHaveBeenCalled();
        });

        describe('Generating options for Interstitial Ads while moving through story elements', function() {
            it('should create an options object for the Interstitial to use to create the ad', function() {
                var options = storyController.generateOptionsForInterstitial(fixture[0]);

                expect(typeof options).toEqual('object');
                expect(options.adUnitId).not.toBe(undefined);
                expect(options.adSize).toEqual('BIGBOX');
                expect(options.tags.pr).toEqual(Pheme.Config.fetchCurrentMarket().adCode);
                expect(options.tags.loc).toEqual(fixture[0].data.ad_codes.loc);
                expect(options.tags.imp).toEqual(fixture[0].data.ad_codes.imp);
                expect(options.tags.page).toEqual(fixture[0].data.ad_codes.page);
                expect(options.tags.ck).toEqual(fixture[0].data.ad_codes.ck);
                expect(options.tags.nk).toEqual(fixture[0].data.ad_codes.nk);
            });

            it('should return undefined if the options passed in are invalid', function() {
                var options = storyController.generateOptionsForInterstitial();

                expect(options).toBe(undefined);
            });
        });

        describe('Getting the direction the user is moving through the carousel', function() {
            it('should return true if the direction is a negative value', function() {
                var goingForward = storyController.isGoingForwardThroughCarousel(-1);

                expect(goingForward).toBe(true);
            });

            it('should return false if the direction is a positive value', function() {
                var goingForward = storyController.isGoingForwardThroughCarousel(1);

                expect(goingForward).toBe(false);
            });
        });

        describe('Retrieving a story in the carousel for the intersitial ad', function() {

            beforeEach(function() {
                carousel = {
                    items:[
                        {
                            internalId: 0
                        },
                        {
                            internalId: 1
                        },
                        {
                            internalId: 2
                        }
                    ],
                    activeIndex: 0,
                    getActiveIndex: function() {
                        return this.activeIndex;
                    },
                    setActiveIndex: function(index) {
                        this.activeIndex = index;
                    },
                    getAt: function(index) {
                        return this.items[index];
                    }
                };
            });

            it('should get the next story in the carousel', function() {
                var nextItemId = storyController.getNextStoryInCarouselForInterstitial(carousel);

                expect(nextItemId).toEqual(carousel.items[2].internalId);
            });

            it('should get the previous story in the carousel', function() {
                carousel.setActiveIndex(2);
                var prevItemId = storyController.getPreviousStoryInCarouselForInterstitial(carousel);

                expect(prevItemId).toEqual(carousel.items[2].internalId);
            });

            it('should get the first story in the carousel if the user is at the beginning of the carousel', function() {
                var prevItemId = storyController.getPreviousStoryInCarouselForInterstitial(carousel);

                expect(prevItemId).toEqual(carousel.items[1].internalId);
            });

            afterEach(function() {
                carousel = undefined;
            });
        });

        afterEach(function() {
            dfpAdHelper = undefined;
        });
    });

    describe('Determining when to display an interstitial ad', function() {
        beforeEach(function() {
            carousel = {
                activeIndex: 0,
                currentStoryId: 1,
                items: [
                    {
                        internalId: 0
                    },
                    {
                        internalId: 1
                    },
                    {
                        internalId: 2
                    },
                    {
                        internalId: 3
                    },
                    {
                        internalId: 4
                    },
                    {
                        internalId: 5
                    },
                    {
                        internalId: 6
                    },
                    {
                        internalId: 7
                    },
                    {
                        internalId: 8
                    },
                    {
                        internalId: 9
                    }
                ],
                insert: function(index, item) {
                    this.items.splice(index, 0, item);
                },
                getAt: function(index) {
                    return this.items[index];
                },
                getItems: function() {
                    return this.items;
                },
                getActiveIndex: function() {
                    return this.activeIndex;
                },
                getActiveItem: function() {
                    return this.items[0];
                },
                setActiveIndex: function(index) {
                    this.activeIndex = index;
                },
                add: function(item) {
                    this.items.push(item);
                }
            };
            store = {
                items: [
                    {
                        internalId: 0,
                        data: {
                            content: [
                                {
                                    title: 'Testing'
                                }
                            ],
                            featured_image_url: 'http://www.test.com/test.png'
                        }
                    },
                    {
                        internalId: 1,
                        data: {
                            content: [
                                {
                                    title: 'Testing 2'
                                }
                            ],
                            featured_image_url: 'http://www.test.com/test.png'
                        }
                    },
                    {
                        internalId: 2,
                        data: {}
                    },
                    {
                        internalId: 3,
                        data: {}
                    },
                    {
                        internalId: 4,
                        data: {}
                    },
                    {
                        internalId: 5,
                        data: {}
                    },
                    {
                        internalId: 6,
                        data: {}
                    },
                    {
                        internalId: 7,
                        data: {}
                    },
                    {
                        internalId: 8,
                        data: {}
                    },
                    {
                        internalId: 9,
                        data: {}
                    }
                ],
                indexOfId: function(id) {
                    for(var i = 0; i < this.items.length; i++) {
                        if(this.items[i].id === id) {
                            return i;
                        } else {
                            return -1;
                        }
                    }
                },
                getAt: function(index) {
                    return this.items[index];
                },
                nextPage: function() {
                    return true;
                },
                getAllCount: function() {
                    return this.items.length;
                },
                find: function(field, value) {
                    if(field === 'id') {
                        field = 'internalId';
                    }

                    for(var i = 0; i < this.items.length; i++) {
                        if(this.items[i][field] === value) {
                            return i;
                            //return this.items[i][field];
                        }
                    }
                }
            };
        });

        it("should find the next story's id given the current id", function() {
            var currentId = 9,
                nextStoryId = storyController.getNextStoryId(currentId, store);

            expect(nextStoryId).toEqual(9);
        });

        it("should return the current id if the next story id cannot be found", function() {
            var currentId = 2,
                nextStoryId = storyController.getNextStoryId(currentId, store);

            expect(nextStoryId).toEqual(3);
        });


        it("should find the previous story's id given the current id", function() {
            var currentId = 2,
                prevStoryId = storyController.getPrevStoryId(currentId, store);

            expect(prevStoryId).toEqual(1);
        });

        it("should return the current id if the previous story id cannot be found", function() {
            var currentId = 0,
                prevStoryId = storyController.getPrevStoryId(currentId, store);

            expect(prevStoryId).toEqual(0);
        });

        it('should load a story into the carousel', function() {
            carousel.items = [];

            var newFixture = fixture;
            newFixture[0].data.id = 0;
            storyController.insertStoryIntoCarousel(newFixture[0].data.id, carousel, store);
            expect(carousel.items.length).toEqual(1);
        });

        it('should return null if story returned from store is not valid', function() {
            var storyResults = storyController.insertStoryIntoCarousel(99, carousel, store);

            expect(storyResults).toBe(null);
        });

        it('should return false if the story is already in the carousel', function() {
            storyController.insertStoryIntoCarousel(0, carousel, store);

            var storyResults = storyController.insertStoryIntoCarousel(0, carousel, store);

            expect(storyResults).not.toBe(0);
        });

        it('should load next page in data set when the starting index is greater than the number of stories in the store minus the number of stories the carousel should contain', function() {
            spyOn(store, 'nextPage');
            app.startIndex = 6;

            carousel.setActiveIndex(11);
            storyController.loadNextPageOfDataIntoStore(carousel, store);

            expect(store.nextPage).toHaveBeenCalled();
            carousel.setActiveIndex(0);
        });

        it('should load an interstitial ad into the carousel', function() {
            spyOn(carousel, 'insert').and.callThrough();
            spyOn(storyController, 'getAd').and.callThrough();

            app.adIndex = 4;
            carousel.setActiveIndex(5);
            carousel.storyId = 0;

            storyController.loadInterstitialAdIntoStoryCarousel(carousel, 'forward');

            expect(carousel.getItems().length).toBeGreaterThan(10);
            expect(carousel.insert).toHaveBeenCalled();
            expect(storyController.getAd).toHaveBeenCalled();
        });

        it('should load an interstitial ad into the carousel while going backwards through the carousel', function() {
            spyOn(carousel, 'insert').and.callThrough();
            spyOn(storyController, 'getAd').and.callThrough();

            app.adIndex = 4;
            carousel.setActiveIndex(5);
            carousel.storyId = 0;

            storyController.loadInterstitialAdIntoStoryCarousel(carousel, 'back');

            expect(carousel.getItems().length).toBeGreaterThan(10);
            expect(carousel.insert).toHaveBeenCalled();
            expect(storyController.getAd).toHaveBeenCalled();
        });

        it("should find the id of the next story in the store", function() {
            var results = storyController.getNextStoryId(store.getAt(0).internalId, store);

            expect(results).toEqual(1);
        });

        it("should find the id of the previous story in the store", function() {
            var results = storyController.getPrevStoryId(store.getAt(1).internalId, store);

            expect(results).toEqual(0);
        });

        it("should insert stories into the carousel properly on the carousel's activation", function() {
            carousel.items = [];
            spyOn(storyController, 'insertStoryIntoCarousel').and.callThrough();
            spyOn(storyController, 'getNextStoryId').and.callThrough();
            spyOn(storyController, 'getPrevStoryId').and.callThrough();

            storyController.setupStoriesForCarouselActivation(store.getAt(0), carousel, store);

            expect(storyController.insertStoryIntoCarousel).toHaveBeenCalled();
            expect(storyController.getNextStoryId).toHaveBeenCalled();
            expect(storyController.getPrevStoryId).toHaveBeenCalled();
            expect(carousel.items.length).toEqual(3);
        });

        afterEach(function() {
            carousel = undefined;
            store = undefined;
        });
    });

    afterEach(function() {
        app = undefined;
        storyController = undefined;
        dfpInst = undefined;
        window.DFPPlugin = undefined;
    });

});
