describe('Breaking News Helper Class', function() {

    describe('Getting number of updated stories', function() {
        it('should return null if no records are supplied', function() {
            var results = Pheme.helpers.BreakingNewsAlerts.getNumberOfUpdatedStories();

            expect(results).toBe(null);
        });

        it('should return number of updated stories', function() {
            var results = Pheme.helpers.BreakingNewsAlerts.getNumberOfUpdatedStories([{data: {updated: true}}]);

            expect(results).toEqual(1);
        });
    });

    describe('Checking for breaking news items', function() {
        it('should return null if no records are supplied', function() {
            var results = Pheme.helpers.BreakingNewsAlerts.checkForBreakingNewsItem();

            expect(results).toBe(null);
        });

        it('should return null if records object has no entries', function() {
            var results = Pheme.helpers.BreakingNewsAlerts.checkForBreakingNewsItem([]);

            expect(results).toBe(null);
        });

        it('should return false if the breaking property is not present', function() {
            var results = Pheme.helpers.BreakingNewsAlerts.checkForBreakingNewsItem([{data: {}}]);

            expect(results).toBe(false);
        });

        it('should return true if the breaking property is present', function() {
            var results = Pheme.helpers.BreakingNewsAlerts.checkForBreakingNewsItem([{data: {breaking: true}}]);

            expect(results).toBe(true);
        });

    });

    describe('Updating Elements with Breaking', function() {
        it('should return null if no records are supplied', function() {
            var results = Pheme.helpers.BreakingNewsAlerts.updateElementsWithBreaking();

            expect(results).toBe(null);
        });

        it('should return null if no updated value is supplied', function() {
            var results = Pheme.helpers.BreakingNewsAlerts.updateElementsWithBreaking('test', undefined);

            expect(results).toBe(null);
        });

        it('should return false without a valid element to update', function() {
            var results = Pheme.helpers.BreakingNewsAlerts.updateElementsWithBreaking('test', 'test');

            expect(results).toBe(false);
        });

        /*it('should return true with a valid element to update', function() {
            var breakingNewsList = Ext.create('Pheme.view.BreakingNewsList');
            var results = Pheme.helpers.BreakingNewsAlerts.updateElementsWithBreaking('BreakingNewsList', 'test');

            expect(results).toBe(true);
        });*/
    });
});
