describe('Common Functions Helper', function() {
    describe('Filtering an array', function() {
        /*it('should return an array with filtered values', function() {
            var arrayA = [0, -1, -1, 0],
                arrayB = [0, 0, -1, 0];

            var newArray = Pheme.helpers.Common.diff(arrayA, arrayB);

            expect(newArray.length).toBeGreaterThan(0);
        });*/
    });
    describe('Opening story in browser', function() {
        beforeEach(function() {
            window = {
                open: function() {
                    return true;
                }
            }
        });

        it('should open story in browser with a valid url', function() {
            spyOn(window, 'open');

            Pheme.helpers.Common.openInBrowser('http://www.google.com');

            expect(window.open).toHaveBeenCalled();
        });

        afterEach(function() {
            window = undefined;
        });
    });

    describe('Switching Views', function() {
        it('should animate active item if os is iOS', function() {
            spyOn(Ext.Viewport, 'animateActiveItem');

            var mainView = Ext.create('Pheme.view.Main');
            Pheme.helpers.Common.switchView(mainView);

            expect(Ext.Viewport.animateActiveItem).toHaveBeenCalled();
        });

        it('should just set the active item if os is Android', function() {
            spyOn(Ext.Viewport, 'setActiveItem');

            Ext.os.is['Android'] = true;
            var mainView = Ext.create('Pheme.view.Main');
            Pheme.helpers.Common.switchView(mainView);

            expect(Ext.Viewport.setActiveItem).toHaveBeenCalled();
        });

        it('should animate the view with supplied options', function() {
            spyOn(Ext.Viewport, 'animateActiveItem').and.callThrough();

            Ext.os.is['iOS'] = true;
            Ext.os.is['Android'] = false;

            var mainView = Ext.create('Pheme.view.Main');
            Pheme.helpers.Common.switchView(mainView, {
                type: 'slide',
                direction: 'left'
            });

            expect(Ext.Viewport.animateActiveItem).toHaveBeenCalled();

            Ext.os.is['iOS'] = false;
            Ext.os.is['Android'] = true;
        });
    });

    describe('Switching elements', function() {
        it('should move the element out with animation if OS is iOS', function() {
            var element = {
                addCls: function(cls) {
                    return this;
                },
                removeCls: function(cls) {
                    return this;
                }
            };

            spyOn(element, 'addCls').and.callThrough();
            spyOn(element, 'removeCls').and.callThrough();

            Ext.os.is['Android'] = false;

            Pheme.helpers.Common.switchElement(element, 'out');

            expect(element.addCls).toHaveBeenCalledWith('out');
            expect(element.removeCls).toHaveBeenCalledWith('in');

            Ext.os.is['Android'] = true;
        });

        it('should move the element in with animation if OS is iOS', function() {
            var element = {
                addCls: function(cls) {
                    return this;
                },
                removeCls: function(cls) {
                    return this;
                }
            };

            spyOn(element, 'addCls').and.callThrough();
            spyOn(element, 'removeCls').and.callThrough();

            Ext.os.is['Android'] = false;

            Pheme.helpers.Common.switchElement(element, false);

            expect(element.addCls).toHaveBeenCalledWith('in');
            expect(element.removeCls).toHaveBeenCalledWith('out');

            Ext.os.is['Android'] = true;
        });

        it('should move the element out with animation if OS is Android', function() {
            var element = {
                addCls: function(cls) {
                    return this;
                },
                removeCls: function(cls) {
                    return this;
                }
            };

            spyOn(element, 'addCls').and.callThrough();
            spyOn(element, 'removeCls').and.callThrough();

            Pheme.helpers.Common.switchElement(element, 'out');

            expect(element.addCls).toHaveBeenCalledWith('out-no-animation');
            expect(element.removeCls).toHaveBeenCalledWith('in in-no-animation');
        });

        it('should move the element in with animation if OS is Android', function() {
            var element = {
                addCls: function(cls) {
                    return this;
                },
                removeCls: function(cls) {
                    return this;
                }
            };

            spyOn(element, 'addCls').and.callThrough();
            spyOn(element, 'removeCls').and.callThrough();

            Pheme.helpers.Common.switchElement(element, false);

            expect(element.addCls).toHaveBeenCalledWith('in-no-animation');
            expect(element.removeCls).toHaveBeenCalledWith('out out-no-animation');
        });
    });
});
