describe('Controller Events Helper', function() {

    var app,
        fixture;

    beforeEach(function() {
        app = Ext.create('Ext.app.Application', {
            name: 'Pheme'
        });

        fixture = {
            data: {
                ad_codes: {
                    ad: "news/local-news/story",
                    ck: "news",
                    loc: "top",
                    nk: "print",
                    page: "story",
                    pr: "oc",
                    sck: [
                        "local-news"
                    ]
                },
                caption: "",
                categories: [],
                length: 1,
                credit: "Greg Thomsen",
                date: "Mon Jul 14 2014 20:57:00 GMT-0700 (PDT)",
                date_sort: "Mon Jul 14 2014 23:59:00 GMT-0700 (PDT)",
                excerpt: "That was a month ago, and since then he has plastered signs around town offering a $150 reward for his four-armed flying friend, but to no avail.",
                featured_image_url: "http://wpmedia.ottawacitizen.com/2014/07/greg-thomsen-lost-his-quad-copter-a-month-ago-sad-days-for-0715-quad-copter.jpg?w=320&h=192&crop=1",
                featured_image_url_x2: "http://wpmedia.ottawacitizen.com/2014/07/greg-thomsen-lost-his-quad-copter-a-month-ago-sad-days-for-0715-quad-copter.jpg?w=640&h=384&crop=1",
                id: 274973,
                last_modified: "Mon Jul 14 2014 20:57:00 GMT-0700 (PDT)",
                main_ad_codes: {
                    ad: "news/index",
                    ck: "news",
                    imp: "news",
                    loc: "top",
                    nk: "print",
                    page: "index"
                },
                no_thumbnail: false,
                permalink: "http://ottawacitizen.com/storyline/hobbyist-must-watch-as-his-800-quad-copter-fly-flies-away",
                post_format: null,
                thumbnail_url: "http://wpmedia.ottawacitizen.com/2014/07/greg-thomsen-lost-his-quad-copter-a-month-ago-sad-days-for-0715-quad-copter.jpg?w=95&h=95&crop=1",
                thumbnail_url_x2: "http://wpmedia.ottawacitizen.com/2014/07/greg-thomsen-lost-his-quad-copter-a-month-ago-sad-days-for-0715-quad-copter.jpg?w=190&h=190&crop=1",
                title: "Hobbyist must watch as his $800 quad-copter fly, flies&nbsp;away",
                updated: false,
                content: []
            }
        };
    });

    describe('Getting a story from the store', function() {
        it('should return undefined from an invalid story id', function() {
            var story = Pheme.helpers.ControllerEvents.getStoryFromStoryStoreById();

            expect(story).toBe(null);
        });

        it('should return a story object when given a valid id', function() {
            var mockStore = {};
            mockStore.getById = function() {
                return fixture;
            };

            var story = Pheme.helpers.ControllerEvents.getStoryFromStoryStoreById(274973, mockStore);

            expect(story).not.toBe(undefined);
        });
    });

    describe('Getting Story Object', function() {
        it('should return null if no arguments are supplied', function() {
            var storyObject = Pheme.helpers.ControllerEvents.getStoryObjectFromEventQueueArguments();

            expect(storyObject).toBe(null);
        });
    });

    describe('Rendering a banner ad', function() {
       it('should call renderBannerAd method given a valid story object', function() {
           spyOn(Pheme.helpers.DfpAds, 'renderBannerAd');

           Pheme.helpers.ControllerEvents.renderBannerAd(fixture);

           expect(Pheme.helpers.DfpAds.renderBannerAd).toHaveBeenCalled();
       });

        it('should return null if no valid story object is found', function() {
            var bannerAd = Pheme.helpers.ControllerEvents.renderBannerAd();

            expect(bannerAd).toBe(null);
        });
    });

    afterEach(function() {
        app = undefined;
    });
});

