describe('DFP Ads Helper', function() {
    var app;

    beforeEach(function() {
        app = Ext.create('Ext.app.Application', {
            name: 'Pheme'
        });

        dfpInstanceMock = function() {};

        dfpInstanceMock.prototype.createBannerAd = function(options, adSuccessCallback, error) {
            return true;
        };

        dfpInstanceMock.prototype.createInterstitialAd = function(options, adSuccessCallback, error) {
            return true;
        };

        dfpInstanceMock.prototype.debugMode = function(options) {
            return true;
        };

        dfpInstanceMock.prototype.removeAd = function(debugObj, callback) {
            return true;
        };

        dfpInst = new dfpInstanceMock();


        Pheme.helpers.DfpAds.setDfpPluginInstance(dfpInst);
    });

    describe('Creating instance of DfpAds Helper', function() {
        it('should call parent when instantiated', function() {
            spyOn(Pheme.helpers.DfpAds, 'callParent');

            Pheme.helpers.DfpAds.init();

            expect(Pheme.helpers.DfpAds.callParent).toHaveBeenCalled();
        });
    });

    describe('Creating a DFP Ad', function() {
        it('should return false without the dfp plugin installed', function() {
            Pheme.helpers.DfpAds.setDfpPluginInstance(undefined);
            var dfpInstance = Pheme.helpers.DfpAds.getDfpPluginInstance(),
                createdAd = Pheme.helpers.DfpAds.createDfpAd();

            expect(dfpInstance).toBe(undefined);
            expect(createdAd).toBe(false);
        });

        it('should use networkID if debug is enabled and networkID has been set', function() {
            Pheme.helpers.DfpAds.setDfpPluginInstance(dfpInst);

            Pheme.helpers.DfpAds.setDebugMode(true);
            Pheme.helpers.DfpAds.setNetworkId('ottawa');

            var bannerAd = Pheme.helpers.DfpAds.createDfpAd();

            expect(bannerAd).toBe(true);
        });

        it('should use test ads if the testAds property is true', function() {
            Pheme.helpers.DfpAds.setTestAds(true);

            var bannerAd = Pheme.helpers.DfpAds.createDfpAd(undefined);

            expect(bannerAd).toBe(true);

            Pheme.helpers.DfpAds.setTestAds(false);
        });
    });

    describe('Creating an Interstitial Ad', function() {

        it('should set its own options for creating the ad if no option parameter is passed in', function() {

            var interstitial = Pheme.helpers.DfpAds.createInterstitialAd();

            expect(interstitial).toBe(true);

        });

        it('should remove previous ads if flag parameter is set', function() {
            spyOn(dfpInst, 'removeAd');

            var interstitial = Pheme.helpers.DfpAds.createInterstitialAd(undefined, true);

            expect(dfpInst.removeAd).toHaveBeenCalled();
        });

        it('should add test tag to ad if testAds is true', function() {
            Pheme.helpers.DfpAds.setTestAds(true);

            var interstitial = Pheme.helpers.DfpAds.createInterstitialAd();
            expect(interstitial).toBe(true);

            Pheme.helpers.DfpAds.setTestAds(false);
        });
    });

    describe('Removing DFP Ads', function() {
        it('should return null when trying to remove ads and there is no DFP Plugin instance', function() {
            Pheme.helpers.DfpAds.setDfpPluginInstance(undefined);
            var deletedAds = Pheme.helpers.DfpAds.removeAds();

            expect(deletedAds).toBe(null);
        });
    });

    describe('Toggling Debug Mode', function() {
        it('should set debug mode to false if it is already set to true', function() {
            Pheme.helpers.DfpAds.setDebugMode(true);

            Pheme.helpers.DfpAds.toggleDebugMode();

            expect(Pheme.helpers.DfpAds.getDebugMode()).toBe(false);
        });

        it('should set debug mode to true if it is already set to false', function() {
            Pheme.helpers.DfpAds.setDebugMode(false);

            Pheme.helpers.DfpAds.toggleDebugMode();

            expect(Pheme.helpers.DfpAds.getDebugMode()).toBe(true);
        });
    });

    describe('Rendering a Banner Ad', function() {
        beforeEach(function() {
            fixture = [{
                data: {
                    ad_codes: {
                        ad: "news/local-news/story",
                        ck: "news",
                        loc: "top",
                        nk: "print",
                        page: "story",
                        pr: "oc",
                        sck: [
                            "local-news"
                        ]
                    },
                    caption: "",
                    categories: [],
                    length: 1,
                    credit: "Greg Thomsen",
                    date: "Mon Jul 14 2014 20:57:00 GMT-0700 (PDT)",
                    date_sort: "Mon Jul 14 2014 23:59:00 GMT-0700 (PDT)",
                    excerpt: "That was a month ago, and since then he has plastered signs around town offering a $150 reward for his four-armed flying friend, but to no avail.",
                    featured_image_url: "http://wpmedia.ottawacitizen.com/2014/07/greg-thomsen-lost-his-quad-copter-a-month-ago-sad-days-for-0715-quad-copter.jpg?w=320&h=192&crop=1",
                    featured_image_url_x2: "http://wpmedia.ottawacitizen.com/2014/07/greg-thomsen-lost-his-quad-copter-a-month-ago-sad-days-for-0715-quad-copter.jpg?w=640&h=384&crop=1",
                    id: 274973,
                    last_modified: "Mon Jul 14 2014 20:57:00 GMT-0700 (PDT)",
                    main_ad_codes: {
                        ad: "news/index",
                        ck: "news",
                        imp: "news",
                        loc: "top",
                        nk: "print",
                        page: "index"
                    },
                    no_thumbnail: false,
                    permalink: "http://ottawacitizen.com/storyline/hobbyist-must-watch-as-his-800-quad-copter-fly-flies-away",
                    post_format: null,
                    thumbnail_url: "http://wpmedia.ottawacitizen.com/2014/07/greg-thomsen-lost-his-quad-copter-a-month-ago-sad-days-for-0715-quad-copter.jpg?w=95&h=95&crop=1",
                    thumbnail_url_x2: "http://wpmedia.ottawacitizen.com/2014/07/greg-thomsen-lost-his-quad-copter-a-month-ago-sad-days-for-0715-quad-copter.jpg?w=190&h=190&crop=1",
                    title: "Hobbyist must watch as his $800 quad-copter fly, flies&nbsp;away",
                    updated: false
                }
            }];
        });

        it('should return null if no story has been passed in', function() {
            var bannerAd = Pheme.helpers.DfpAds.renderBannerAd();

            expect(bannerAd).toBe(null);
        });

        it('should set a default background color if none is passed in', function() {
            spyOn(Pheme.helpers.DfpAds, 'createDfpAd');

            Pheme.helpers.DfpAds.setTestAds(false);

            Pheme.helpers.DfpAds.renderBannerAd(fixture[0], undefined, true, false);

            expect(Pheme.helpers.DfpAds.createDfpAd).toHaveBeenCalledWith(
                {
                    'adSize': 'BANNER',
                    'tags': {
                        'pr': Pheme.Config.fetchCurrentMarket().adCode, 'loc': fixture[0].data.ad_codes.loc, 'imp': fixture[0].data.ad_codes.imp,
                        'page': fixture[0].data.ad_codes.page,
                        'ck': fixture[0].data.ad_codes.ck,
                        'nk': fixture[0].data.ad_codes.nk,
                        'sck' : fixture[0].data.ad_codes.sck
                    },
                    'backgroundColor': '#252525',
                    'adUnitId': '/3081/' + Pheme.Config.fetchCurrentMarket().adCode + '.aroid/news/local-news/story'
                }, true
            );
        });

        it('should set the test ads property for the ad if testAds is true', function() {
            spyOn(Pheme.helpers.DfpAds, 'createDfpAd');
            Pheme.helpers.DfpAds.setTestAds(true);
            Pheme.helpers.DfpAds.renderBannerAd(fixture[0], undefined, true, false);

            expect(Pheme.helpers.DfpAds.createDfpAd).toHaveBeenCalledWith(
                {
                    'adSize': 'BANNER',
                    'tags': {
                        'pr': Pheme.Config.fetchCurrentMarket().adCode, 'loc': fixture[0].data.ad_codes.loc, 'imp': fixture[0].data.ad_codes.imp,
                        'page': fixture[0].data.ad_codes.page,
                        'ck': fixture[0].data.ad_codes.ck,
                        'nk': fixture[0].data.ad_codes.nk,
                        'sck' : fixture[0].data.ad_codes.sck,
                        'test': 'yes'
                    },

                    'backgroundColor': '#252525',
                    'adUnitId': '/3081/' + Pheme.Config.fetchCurrentMarket().adCode + '.aroid/news/local-news/story'
                }, true
            );
        });

        it('should use the main ad codes to create the ad if the story has main ad codes and useMainAdCodes is true', function() {
            spyOn(Pheme.helpers.DfpAds, 'createDfpAd');
            Pheme.helpers.DfpAds.setTestAds(false);
            Pheme.helpers.DfpAds.renderBannerAd(fixture[0], '#252525', true, true);

            expect(Pheme.helpers.DfpAds.createDfpAd).toHaveBeenCalledWith(
                {
                    'adSize': 'BANNER',
                    'tags': {
                        'pr': Pheme.Config.fetchCurrentMarket().adCode, 'loc': fixture[0].data.main_ad_codes.loc, 'imp': fixture[0].data.main_ad_codes.imp,
                        'page': fixture[0].data.main_ad_codes.page,
                        'ck': fixture[0].data.main_ad_codes.ck,
                        'nk': fixture[0].data.main_ad_codes.nk
                    },
                    'backgroundColor': '#252525',
                    'adUnitId': '/3081/' + Pheme.Config.fetchCurrentMarket().adCode + '.aroid/news/index'
                }, true
            );
        });

        afterEach(function() {
            fixture = undefined;
        });
    });

    describe('Toggling Test Ads', function() {
        it('should set test ads to false if it is already set to true', function() {
            Pheme.helpers.DfpAds.setTestAds(true);

            Pheme.helpers.DfpAds.toggleTestAds();

            expect(Pheme.helpers.DfpAds.getTestAds()).toBe(false);
        });

        it('should set test ads to true if it is already set to false', function() {
            Pheme.helpers.DfpAds.setTestAds(false);

            Pheme.helpers.DfpAds.toggleTestAds();

            expect(Pheme.helpers.DfpAds.getTestAds()).toBe(true);
        });
    });

    describe('Ad Creation Callbacks', function() {
        it('should make the call to requestAd', function() {
            spyOn(dfpInst, 'requestAd');

            Pheme.helpers.DfpAds.adCreationSuccessfulCallback();

            expect(dfpInst.requestAd).toHaveBeenCalled();
        });

        it('should just log out error message if there is an error', function() {
            spyOn(Pheme.helpers.DfpAds, 'logAdCreationResult');

            Pheme.helpers.DfpAds.adCreationFailureCallback();

            expect(Pheme.helpers.DfpAds.logAdCreationResult).toHaveBeenCalled();
        });
    });

    afterEach(function() {
        app = undefined;
        Pheme.helpers.DfpAds.setDfpPluginInstance(undefined);
    });
});
