describe('Polar Ads Helper', function() {
	describe('initializing the SDK', function() {
		it('should load the SDK so that nativeads-plugin is present', function() {
            Pheme.helpers.PolarAdsHelper.initSdk();
			var polarSdk = document.getElementById('nativeads-plugin');
			expect(polarSdk).not.toEqual(null);
		});
	});

    describe('setting up the SDK', function() {

        var queue;

        beforeEach(function() {
            Pheme.helpers.PolarAdsHelper.setupSdk();
            queue = Pheme.helpers.PolarAdsHelper.getQueue();
        });

        it('should set single page app mode to true', function() {
            expect(queue[0]).toContain("enableSPAMode");
        });

        it('should set the property key', function() {
            expect(queue[1]).toContain("setPropertyID");
        });

        it('should set a default property key if one is not supplied', function() {
            expect(queue[1]).toContain("NA-TEST-11235814");
        });
    });

    describe('requesting a native ad', function() {

        beforeEach(function() {
            Pheme.helpers.PolarAdsHelper.initSdk();
        });

        it('should return undefined if no unit data is passed in', function() {
            var testAd = Pheme.helpers.PolarAdsHelper.requestAd();
            expect(testAd).toBeUndefined();
        });

        it('should return true given object containing ad data', function() {
            var adUnit = {
                "server" : "dfp",
                "id"     : "/7814368/NativeAdSamples",
                "size"   : "1x1"
            };

            var testAd = Pheme.helpers.PolarAdsHelper.requestAd(adUnit);

            expect(testAd).toBe(true);
        });

        describe('the onFill callback method', function() {
            var storyStore;

            it('should add a Story model to the Stories store', function() {

                storyStore = Ext.getStore('Stories');
                var model = Ext.create('Pheme.model.Story'),
                    mockReturnAd = {
                        adTag: {
                        },
                        model: {
                            author: {
                                name: "Wystan"
                            },
                            content: '<div class="me-content-container"><div class="me-content-body"><div id="pna:article:f5676b37667f449987550f649080eafb:body"><p>Prevent many people who are in dire need of debt relief from filing bankruptcy.</p> <p>The myths and misconceptions surrounding personal bankruptcy prevent many people who are in dire need of debt relief from filing bankruptcy. However, many of these popular beliefs about personal bankruptcy are far from true. Read on to learn the truth about filing bankruptcy.</p>',
                            title: "7 Myths About Personal Bankruptcy",
                            id: "bc579e125f7643ff9521ea830048c48c"
                        }
                    };

                expect(storyStore.getCount()).toEqual(10);
                Pheme.helpers.PolarAdsHelper.onFill(mockReturnAd, 4, storyStore, model);
                expect(storyStore.getCount()).toEqual(11);
                expect(storyStore.getAt(4).data.id).toEqual(mockReturnAd.model.id);
            });

            afterEach(function() {
                storyStore.removeAt(4);
            });
        });
    });

    describe('Getting correct position for Ad', function() {
        it('should return two dates from the store', function() {

            var dates = Pheme.helpers.PolarAdsHelper.getDateRangeFromIndex(4);
            expect(dates.length).toEqual(2);
            expect(dates[0] instanceof Date).toBe(true);
            expect(dates[1] instanceof Date).toBe(true);
        });

        it('should set date of model based on the index of the store to be inserted into', function() {
            var model = Ext.create('Pheme.model.Story'),
                store = Ext.getStore('Stories');
                index = 4;
                dateRange = Pheme.helpers.PolarAdsHelper.getDateRangeFromIndex(4, store);

            var newModel = Pheme.helpers.PolarAdsHelper.setDateBasedOnIndexInsert(index, model, dateRange);
            expect(model.data.date_sort.getTime()).toBeGreaterThan(dateRange[0].getTime());
            expect(model.data.date_sort.getTime()).toBeLessThan(dateRange[1].getTime());
        });
    });
});
